unit COMDirect_Main;

interface

// exported functions
procedure GetPortsAvail(var LPT, COM, JOY: boolean); export; stdCall;
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);  export; stdCall;
procedure GetShortDesc(var s: ShortString); export; stdCall;
procedure GetLongDesc(var s: ShortString); export; stdCall;
procedure GetCANFullScale(var d: double); export; stdCall;
procedure GetCANResolution(var i: byte); export; stdCall;
procedure GetAnalogInputsCapacity(var i: byte); export; stdCall;
procedure GetDigitalInputsCapacity(var i: byte); export; stdCall;
procedure GetInputsLabel(var ar: array of ShortString); export; stdCall;
procedure InterfaceDetected(var b: boolean); export; stdCall;
procedure RunTest(var bResult: boolean); export; stdCall;
procedure InitHard(var bResult: boolean); export; stdCall;
procedure ResetHard(var bResult: boolean); export; stdCall;
procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean); export; stdCall;

// internal functions only
procedure COM_Open;
procedure COM_Close;
procedure Delay(ms: integer);
function  COM_ReadAllValues: boolean;
function  COM_WriteAllValues: boolean;
procedure SetError(bState: boolean; sErr: shortString);


const
  MaxAnalogInputs = 0;
  MaxDigitalInputs = 4;
  DataType_Byte = 0;
  DataType_Word = 1;
  DataType_DWord = 2;

  // ports addresses
  //LPT1PortBaseAddr = $378;        // 888d
  //LPT2PortBaseAddr = $278;        // 632d
  COM1PortBaseAddr = $3F8;        // 1016d
  COM2PortBaseAddr = $2F8;        // 760d
  COM3PortBaseAddr = $3E8;        // 1000d
  COM4PortBaseAddr = $2E8;        // 744d
  //JOY1PortBaseAddr = $201;

var
  FHandle: integer; // to handle com port

  DataType: byte = DataType_DWord;

  LastErrorOccured: shortString;
  //LPTActivePortBaseAddr: word;
  COMActivePortBaseAddr: word;
  //JOYActivePortBaseAddr: word;
  KeepOriginalState: boolean;

  I_CTS, I_DSR, I_RI, I_DCD: boolean;
  O_DTR, O_RTS, O_TX: boolean;

resourceString
  sErrCOMRead = 'Erreur de lecture sur [COM]';

implementation

uses
  Messages,
  Windows,
  SysUtils,
  ZLPortIO;



procedure COM_Open;
var
  sPort: String;
begin
  FHandle := INVALID_HANDLE_VALUE;
  case COMActivePortBaseAddr of
    COM1PortBaseAddr: sPort := 'COM1';
    COM2PortBaseAddr: sPort := 'COM2';
    COM3PortBaseAddr: sPort := 'COM3';
    COM4PortBaseAddr: sPort := 'COM4';
  end;
  FHandle := CreateFile(
    PChar(sPort),
    GENERIC_READ or GENERIC_WRITE,
    0,
    nil,
    OPEN_EXISTING,
    FILE_FLAG_OVERLAPPED,
    0);
end;

procedure COM_Close;
begin
  if FHandle <> INVALID_HANDLE_VALUE then
    CloseHandle(FHandle);
end;

procedure Delay(ms: integer);
var
  t0, t1: integer;
begin
  t0 := getTickCount;
  repeat
    t1 := getTickCount;
  until (t1 - t0) > ms;
end;

procedure GetPortsAvail(var LPT, COM, JOY: boolean);
begin
  LPT := false;
  COM := true;
  JOY := false;
end;

procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);
begin
  //LPTActivePortBaseAddr := LPTAddr;
  COMActivePortBaseAddr := COMAddr;
  //JOYActivePortBaseAddr := JOYAddr;
  COM_Close;
  COM_Open;
end;

{
************************************************************************
  serial port
************************************************************************
}

function COM_ReadAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  if FHandle = INVALID_HANDLE_VALUE then
    exit;

  Addr := COMActivePortBaseAddr + 6;

  // read value on port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;

  // read status value on port if system IS WinNT
  end else begin
    try
      Data := ZLIOPortRead(Addr, DataType);
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_DCD := ((Data and 128) = 128);
  I_RI := ((Data and 64) = 64);
  I_DSR := ((Data and 32) = 32);
  I_CTS := ((Data and 16) = 16);

  result := true;
end;

function COM_WriteAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
  i: byte;
begin
  if FHandle = INVALID_HANDLE_VALUE then
    exit;

  for i := 1 to 2 do
  begin
    Data := 0;
    case i of

      // set Modem Commands address and Value
      1 :
        begin
          Addr := COMActivePortBaseAddr + 3;
          if O_TX then Data := Data + 64;    // TX = bit 6
        end;

      // set Line Commands address and Value
      2 :
        begin
          Addr := COMActivePortBaseAddr + 4;
          if O_DTR then Data := Data + 1;    // DTR = bit 0
          if O_RTS then Data := Data + 2;    // RTS = bit 1
        end;

    end;

    // write value on port if system IS NOT WinNT
    if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
      try

        Case DataType of
          DataType_Byte :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, al
            end;
          DataType_Word :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, ax
            end;
          DataType_DWord:
            asm
              mov edx, Addr
              mov eax, Data
              out dx, eax
            end;
         end;
        {
        asm
          mov dx, Addr
          mov ax, Data
          out dx, al
        end;
        }

      except
        result := false;
        exit;
      end;

    // write value on port if system IS WinNT
    end else begin
      try
        ZLIOPortWrite(Addr, DataType, Data);
      except
        result := false;
        exit;
      end;

    end  // if then else

  end;  // for

  result := true;
end;

procedure GetShortDesc(var s: ShortString);
begin
  s := '4 Entres Num. sur port COM';
end;

procedure GetLongDesc(var s: ShortString);
begin
  s := '4 entres numriques directes sur port srie';
end;

procedure GetCANFullScale(var d: double);
begin
  d := 0.0;
end;

procedure GetCANResolution(var i: byte);
begin
  i := 0;
end;

procedure GetAnalogInputsCapacity(var i: byte);
begin
  i := MaxAnalogInputs;
end;

procedure GetDigitalInputsCapacity(var i: byte);
begin
  i := MaxDigitalInputs;
end;

procedure GetInputsLabel(var ar: array of ShortString);
begin
  // labels for direct digital inputs
  ar[64] := 'CTS';
  ar[65] := 'DSR';
  ar[66] := 'RI';
  ar[67] := 'DCD';
end;

procedure InterfaceDetected(var b: boolean);
begin
  // do the job
  b := true;
end;

procedure RunTest(var bResult: boolean);
var
  Old_DTR, Old_RTS, Old_TX: boolean;
  i: integer;
begin

  // save initial values
  Old_DTR := O_DTR;
  Old_RTS := O_RTS;
  Old_TX := O_TX;

  // run test
  O_DTR := false;
  O_RTS := false;
  O_TX := false;
  try
    for i:= 0 to 4 do
    begin
      O_DTR := (i = 1);
      O_RTS := (i = 2);
      O_TX := (i = 3);
      bResult := Com_WriteAllValues;
      delay(100);
    end;

    // restore initial values
    O_DTR := Old_DTR;
    O_RTS := Old_RTS;
    O_TX := Old_TX;
    bResult := Com_WriteAllValues;
  except
    bResult := false;
  end;
end;

procedure InitHard(var bResult: boolean);
begin
  O_DTR := false;
  O_RTS := false;
  O_TX := false;
  bResult := COM_WriteAllValues;
end;

procedure ResetHard(var bResult: boolean);
begin
  // do the job
  bResult := true;
end;

procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean);
var
  ReadOK: boolean;
begin
  try
    ReadOK := Com_ReadAllValues;
    arD[0] := I_CTS;
    arD[1] := I_DSR;
    arD[2] := I_RI;
    arD[3] := I_DCD;
  except
    ReadOK := false;
  end;
  bResult := ReadOK;
  if not ReadOK then
    SetError(true, sErrCOMRead);
end;

procedure SetError(bState: boolean; sErr: shortString);
begin
  if bState then
    LastErrorOccured := sErr
  else
    LastErrorOccured := '';
end;


initialization

finalization
  COM_Close;


end.
