unit LPT64D_Main;

interface

// exported functions
procedure GetPortsAvail(var LPT, COM, JOY: boolean); export; stdCall;
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: integer);  export; stdCall;
procedure GetShortDesc(var s: ShortString); export; stdCall;
procedure GetLongDesc(var s: ShortString); export; stdCall;
procedure GetCANFullScale(var d: double); export; stdCall;
procedure GetCANResolution(var i: byte); export; stdCall;
procedure GetAnalogInputsCapacity(var i: byte); export; stdCall;
procedure GetDigitalInputsCapacity(var i: byte); export; stdCall;
procedure GetInputsLabel(var ar: array of ShortString); export; stdCall;
procedure InterfaceDetected(var b: boolean); export; stdCall;
procedure RunTest(var bResult: boolean); export; stdCall;
procedure InitHard(var bResult: boolean); export; stdCall;
procedure ResetHard(var bResult: boolean); export; stdCall;
procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean); export; stdCall;

// internal functions only
function  LPT_ReadAllValues: boolean;
function  LPT_WriteAllValues: boolean;
procedure SetError(bState: boolean; sErr: shortString);

const
  MaxAnalogInputs = 0;
  MaxDigitalInputs = 64;
  DataType_Byte = 0;
  DataType_Word = 1;
  DataType_DWord = 2;

var
  DataType: byte = DataType_DWord;

  LastErrorOccured: shortString;
  LPTActivePortBaseAddr: integer;
  //COMActivePortBaseAddr: integer;
  //JOYActivePortBaseAddr: integer;
  KeepOriginalState: boolean;

  I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean;
  O_Strobe, O_Autofeed, O_Init, O_Select: boolean;
  O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7: boolean;

resourceString
  sErrLPTRead = 'Erreur de lecture sur [LPT]';


implementation

uses
  Windows,
  SysUtils,
  ZLPortIO;


procedure GetPortsAvail(var LPT, COM, JOY: boolean);
begin
  LPT := true;
  COM := false;
  JOY := false;
end;

procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: integer);
begin
  LPTActivePortBaseAddr := LPTAddr;
  //COMActivePortBaseAddr := COMAddr;
  //JOYActivePortBaseAddr := JOYAddr;
end;

function  LPT_ReadAllValues: boolean;
var
  StatusInputValue: byte;
  StatusAddr: word;
begin
  StatusAddr := LPTActivePortBaseAddr + 1;      // Status register address

  // init status (only used if error occurs during read)
  I_Busy := false;
  I_Ack := false;
  I_PaperEnd := false;
  I_Select := false;
  I_Error := false;

  // read status on port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      asm
        mov dx, StatusAddr
        in al, dx
        mov StatusInputValue, al
      end;
    except
      result := false;
      exit;
    end;

  // read status value on port if system IS WinNT
  end else begin
    try
      StatusInputValue := ZLIOPortRead(StatusAddr, 1);
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_Busy := (((StatusInputValue and 128) div 128) = 1);
  I_Ack := (((StatusInputValue and 64) div 64) = 1);
  I_PaperEnd := (((StatusInputValue and 32) div 32) = 1);
  I_Select := (((StatusInputValue and 16) div 16) = 1);
  I_Error := (((StatusInputValue and 8) div 8) = 1);

  // invert state ?
  if not KeepOriginalState then begin
    // I_Busy : keep original state
    I_Ack := not I_Ack;
    I_PaperEnd := not I_PaperEnd;
    I_Select := not I_Select;
    I_Error := not I_Error;
  end;

  result := true;
end;

function LPT_WriteAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
  i: byte;
begin
  for i := 1 to 2 do
  begin
    Data := 0;
    case i of

      // set Data Commands address and Value
      1 :
        begin
          Addr := LPTActivePortBaseAddr;
          if O_D0 then Data := Data + 1;
          if O_D1 then Data := Data + 2;
          if O_D2 then Data := Data + 4;
          if O_D3 then Data := Data + 8;
          if O_D4 then Data := Data + 16;
          if O_D5 then Data := Data + 32;
          if O_D6 then Data := Data + 64;
          if O_D7 then Data := Data + 128;
        end;

      // set Status Commands address and Value
      2 :
        begin
          Addr := LPTActivePortBaseAddr + 2;
          if not O_Strobe then Data := Data + 1;
          if not O_Autofeed then Data := Data + 2;
          if O_Init then Data := Data + 4;
          if not O_Select then Data := Data + 8;
        end;

    end;

    // write value on port if system IS NOT WinNT
    if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
      try
        Case DataType of
          DataType_Byte :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, al
            end;
          DataType_Word :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, ax
            end;
          DataType_DWord:
            asm
              mov edx, Addr
              mov eax, Data
              out dx, eax
            end;
         end;
        {
        asm
          mov dx, Addr
          mov ax, Data
          out dx, al
        end;
        }
      except
        result := false;
        exit;
      end;

    // write value on port if system IS WinNT
    end else begin
      try
        ZLIOPortWrite(Addr, DataType, Data);
      except
        result := false;
        exit;
      end;

    end;  // if then else

  end;  // for

  result := true;
end;

procedure GetShortDesc(var s: ShortString);
begin
  s := '64 Entres Num. sur port LPT';
end;

procedure GetLongDesc(var s: ShortString);
begin
  s := 'Interface 64 entres numriques base sur des composants courants';
end;

procedure GetCANFullScale(var d: double);
begin
  d := 0.0;
end;

procedure GetCANResolution(var i: byte);
begin
  i := 0;
end;

procedure GetAnalogInputsCapacity(var i: byte);
begin
  i := MaxAnalogInputs;
end;

procedure GetDigitalInputsCapacity(var i: byte);
begin
  i := MaxDigitalInputs;
end;

procedure GetInputsLabel(var ar: array of ShortString);
var
  i: integer;
  Min, Max: byte;
begin
  // labels for digital inputs (none for analog)
  Min := 64;
  GetDigitalInputsCapacity(Max);
  Max := Max + Min;
  for i := Min to pred(Max) do
    ar[i] := format('Entre num. #%.3d', [i - 64 + 1]);
end;

procedure InterfaceDetected(var b: boolean);
var
  bBusy, bAck, bPaperEnd, bSelectI, bError: boolean;
begin
{
  try
    ParallelPort_ReadStatusValues(ActivePortBaseAddr,
      bBusy, bAck, bPaperEnd, bSelectI, bError);
    b := true;
  except
    b := false;
  end;
}
  b := true;
end;

procedure RunTest(var bResult: boolean);
begin
  // read one time on LPT port
  // this test is enough to detect problems
  try
    bResult := LPT_ReadAllValues;
  except
    bResult := false;
  end;
end;

procedure InitHard(var bResult: boolean);
begin
  O_D0 := false;
  O_D1 := false;
  O_D2 := false;
  O_D3 := false;
  O_D4 := false;
  O_D5 := false;
  O_D6 := false;
  O_D7 := false;
  O_Strobe := false;
  O_Autofeed := false;
  O_Init := false;
  O_Select := false;
  bResult := LPT_WriteAllValues;
end;

procedure ResetHard(var bResult: boolean);
begin
  // do nothing
  bResult := true;
end;

procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean);
var
  ReadOK: boolean;
begin
  // I0=Busy, I1=Ack, I2=PaperEnd, I3=SelectI, I4=Error

  // use interface access procedure to read all 64 inputs
  // at he moment, read only 5 inputs

  try
    ReadOK := LPT_ReadAllValues;
    arD[0] := I_Busy;
    arD[1] := I_Ack;
    arD[2] := I_PaperEnd;
    arD[3] := I_Select;
    arD[4] := I_Error;

  except
    ReadOK := false;
  end;
  bResult := ReadOK;
  if not ReadOK then
    SetError(true, sErrLPTRead);
end;

procedure SetError(bState: boolean; sErr: shortString);
begin
  if bState then
    LastErrorOccured := sErr
  else
    LastErrorOccured := '';
end;


end.
