unit LPTDirect_Main;

interface

// exported functions
procedure GetPortsAvail(var LPT, COM, JOY: boolean); export; stdCall;
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);  export; stdCall;
procedure GetShortDesc(var s: ShortString); export; stdCall;
procedure GetLongDesc(var s: ShortString); export; stdCall;
procedure GetCANFullScale(var d: double); export; stdCall;
procedure GetCANResolution(var i: byte); export; stdCall;
procedure GetAnalogInputsCapacity(var i: byte); export; stdCall;
procedure GetDigitalInputsCapacity(var i: byte); export; stdCall;
procedure GetInputsLabel(var ar: array of ShortString); export; stdCall;
procedure InterfaceDetected(var b: boolean); export; stdCall;
procedure InitHard(var bResult: boolean); export; stdCall;
procedure ResetHard(var bResult: boolean); export; stdCall;
procedure RunTest(var bResult: boolean); export; stdCall;
procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean); export; stdCall;

// internal functions only
procedure Delay(ms: integer);
function  LPT_ReadAllValues: boolean;
function  LPT_WriteAllValues: boolean;
procedure SetError(bState: boolean; sErr: shortString);

const
  MaxAnalogInputs = 0;
  MaxDigitalInputs = 5;
  DataType_Byte = 0;
  DataType_Word = 1;
  DataType_DWord = 2;

var

  DataType: byte = DataType_DWord;

  // ports addresses
  //LPT1PortBaseAddr = $378;        // 888d
  //LPT2PortBaseAddr = $278;        // 632d
  //COM1PortBaseAddr = $3F8;        // 1016d
  //COM2PortBaseAddr = $2F8;        // 760d
  //COM3PortBaseAddr = $3E8;        // 1000d
  //COM4PortBaseAddr = $2E8;        // 744d
  //JOY1PortBaseAddr = $201;


  LastErrorOccured: shortString;
  LPTActivePortBaseAddr: word;
  //COMActivePortBaseAddr: word;
  //JOYActivePortBaseAddr: word;
  KeepOriginalState: boolean;

  I_Busy, I_Ack, I_PaperEnd, I_Select, I_Error: boolean;
  O_Strobe, O_Autofeed, O_Init, O_Select: boolean;
  O_D0, O_D1, O_D2, O_D3, O_D4, O_D5, O_D6, O_D7: boolean;

resourceString
  sErrLPTRead = 'Erreur de lecture sur [LPT]';


implementation

uses
  Windows,
  SysUtils,
  ZLPortIO;


procedure Delay(ms: integer);
var
  t0, t1: integer;
begin
  t0 := getTickCount;
  repeat
    t1 := getTickCount;
  until (t1 - t0) > ms;
end;

procedure GetPortsAvail(var LPT, COM, JOY: boolean);
begin
  LPT := true;
  COM := false;
  JOY := false;
end;

procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);
begin
  LPTActivePortBaseAddr := LPTAddr;
  //COMActivePortBaseAddr := COMAddr;
  //JOYActivePortBaseAddr := JOYAddr;
  //showmessage(inttostr(LPTActivePortBaseAddr));  // needs dialogs unit
end;

function  LPT_ReadAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  Addr := LPTActivePortBaseAddr + 1;    // Status register address

  // init status (only used if error occurs during read)
  I_Busy := false;
  I_Ack := false;
  I_PaperEnd := false;
  I_Select := false;
  I_Error := false;

  // read status on port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;

  // read status value on port if system IS WinNT
  end else begin
    try
      Data := ZLIOPortRead(Addr, DataType);
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_Busy := (((Data and 128) div 128) = 1);
  I_Ack := (((Data and 64) div 64) = 1);
  I_PaperEnd := (((Data and 32) div 32) = 1);
  I_Select := (((Data and 16) div 16) = 1);
  I_Error := (((Data and 8) div 8) = 1);

  // invert state ?
  KeepOriginalState := false;         // yes !!!
  if not KeepOriginalState then begin
    // I_Busy : keep original state
    I_Ack := not I_Ack;
    I_PaperEnd := not I_PaperEnd;
    I_Select := not I_Select;
    I_Error := not I_Error;
  end;

    {
    // debug
    I_Busy := false;
    I_Ack := false;
    I_PaperEnd := false;
    I_Select := false;
    I_Error := true;
    }

  result := true;
end;

function LPT_WriteAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
  i: byte;
begin
  for i := 1 to 2 do
  begin
    Data := 0;
    case i of

      // set Data Commands address and Value
      1 :
        begin
          Addr := LPTActivePortBaseAddr;
          if O_D0 then Data := Data + 1;
          if O_D1 then Data := Data + 2;
          if O_D2 then Data := Data + 4;
          if O_D3 then Data := Data + 8;
          if O_D4 then Data := Data + 16;
          if O_D5 then Data := Data + 32;
          if O_D6 then Data := Data + 64;
          if O_D7 then Data := Data + 128;
        end;

      // set Status Commands address and Value
      2 :
        begin
          Addr := LPTActivePortBaseAddr + 2;
          if not O_Strobe then Data := Data + 1;
          if not O_Autofeed then Data := Data + 2;
          if O_Init then Data := Data + 4;
          if not O_Select then Data := Data + 8;
        end;

    end;

    // write value on port if system IS NOT WinNT
    if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
      try
        Case DataType of
          DataType_Byte :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, al
            end;
          DataType_Word :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, ax
            end;
          DataType_DWord:
            asm
              mov edx, Addr
              mov eax, Data
              out dx, eax
            end;
         end;
        {
        asm
          mov dx, Addr
          mov ax, Data
          out dx, al
        end;
        }
      except
        result := false;
        exit;
      end;

    // write value on port if system IS WinNT
    end else begin
      try
        ZLIOPortWrite(Addr, DataType, Data);
      except
        result := false;
        exit;
      end;

    end;  // if then else

  end;  // for

  result := true;
end;

procedure GetShortDesc(var s: ShortString);
begin
  s := '5 Entres Num. sur port LPT';
end;

procedure GetLongDesc(var s: ShortString);
begin
  s := '5 entres numriques directes sur port imprimante';
end;

procedure GetCANFullScale(var d: double);
begin
  d := 0.0;
end;

procedure GetCANResolution(var i: byte);
begin
  i := 0;
end;

procedure GetAnalogInputsCapacity(var i: byte);
begin
  i := MaxAnalogInputs;
end;

procedure GetDigitalInputsCapacity(var i: byte);
begin
  i := MaxDigitalInputs;
end;

procedure GetInputsLabel(var ar: array of ShortString);
begin
  // labels for direct digital inputs
  ar[64] := 'Busy';
  ar[65] := 'Ack';
  ar[66] := 'PaperEnd';
  ar[67] := 'Select';
  ar[68] := 'Error';
end;

procedure InterfaceDetected(var b: boolean);
var
  bBusy, bAck, bPaperEnd, bSelectI, bError: boolean;
begin
{
  try
    ParallelPort_ReadStatusValues(ActivePortBaseAddr,
      bBusy, bAck, bPaperEnd, bSelectI, bError);
    b := true;
  except
    b := false;
  end;
}
  b := true;
end;

procedure RunTest(var bResult: boolean);
var
  Old_D0, Old_D1, Old_D2, Old_D3, Old_D4, Old_D5, Old_D6, Old_D7: boolean;
  Old_Strobe, Old_Autofeed, Old_Init, Old_Select: boolean;
  i: integer;
begin

  // save initial values
  Old_D0 := O_D0;
  Old_D1 := O_D1;
  Old_D2 := O_D2;
  Old_D3 := O_D3;
  Old_D4 := O_D4;
  Old_D5 := O_D5;
  Old_D6 := O_D6;
  Old_D7 := O_D7;
  Old_Strobe := O_Strobe;
  Old_Autofeed := O_Autofeed;
  Old_Init := O_Init;
  Old_Select := O_Select;

  // run test
  O_D0 := false;
  O_D1 := false;
  O_D2 := false;
  O_D3 := false;
  O_D4 := false;
  O_D5 := false;
  O_D6 := false;
  O_D7 := false;
  O_Strobe := false;
  O_Autofeed := false;
  O_Init := false;
  O_Select := false;
  try
    for i := 0 to 13 do
    begin
      O_D0 := (i = 1);
      O_D1 := (i = 2);
      O_D2 := (i = 3);
      O_D3 := (i = 4);
      O_D4 := (i = 5);
      O_D5 := (i = 6);
      O_D6 := (i = 7);
      O_D7 := (i = 8);
      O_Strobe := (i = 9);
      O_Autofeed := (i = 10);
      O_Init := (i = 11);
      O_Select := (i = 12);
      bResult := LPT_WriteAllValues;
      delay(100);
    end;

    // restore initial values
    O_D0 := Old_D0;
    O_D1 := Old_D1;
    O_D2 := Old_D2;
    O_D3 := Old_D3;
    O_D4 := Old_D4;
    O_D5 := Old_D5;
    O_D6 := Old_D6;
    O_D7 := Old_D7;
    O_Strobe := Old_Strobe;
    O_Autofeed := Old_Autofeed;
    O_Init := Old_Init;
    O_Select := Old_Select;
    bResult := LPT_WriteAllValues;
  except
    bResult := false;
  end;
end;

procedure InitHard(var bResult: boolean);
begin
  O_D0 := false;
  O_D1 := false;
  O_D2 := false;
  O_D3 := false;
  O_D4 := false;
  O_D5 := false;
  O_D6 := false;
  O_D7 := false;
  O_Strobe := false;
  O_Autofeed := false;
  O_Init := false;
  O_Select := false;
  bResult := LPT_WriteAllValues;
end;

procedure ResetHard(var bResult: boolean);
begin
  // just init outputs, no hardware to reset
  InitHard(bResult);
end;

procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean);
var
  ReadOK: boolean;
begin
  // I0=Busy, I1=Ack, I2=PaperEnd, I3=SelectI, I4=Error
  try
    ReadOK := LPT_ReadAllValues;
    arD[0] := I_Busy;
    arD[1] := I_Ack;
    arD[2] := I_PaperEnd;
    arD[3] := I_Select;
    arD[4] := I_Error;
  except
    ReadOK := false;
  end;
  bResult := ReadOK;
  if not ReadOK then
    SetError(true, sErrLPTRead);
end;

procedure SetError(bState: boolean; sErr: shortString);
begin
  if bState then
    LastErrorOccured := sErr
  else
    LastErrorOccured := '';
end;


end.
