unit TLC1543_Main;

interface

uses
  Forms,
  OoMisc, AdPort;

// exported functions
procedure GetPortsAvail(var LPT, COM, JOY: boolean); export; stdCall;
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);  export; stdCall;
procedure GetShortDesc(var s: ShortString); export; stdCall;
procedure GetLongDesc(var s: ShortString); export; stdCall;
procedure GetCANFullScale(var d: double); export; stdCall;
procedure GetCANResolution(var i: byte); export; stdCall;
procedure GetAnalogInputsCapacity(var i: byte); export; stdCall;
procedure GetDigitalInputsCapacity(var i: byte); export; stdCall;
procedure GetInputsLabel(var ar: array of ShortString); export; stdCall;
procedure InterfaceDetected(var b: boolean); export; stdCall;
procedure RunTest(var bResult: boolean); export; stdCall;
procedure InitHard(var bResult: boolean); export; stdCall;
procedure ResetHard(var bResult: boolean); export; stdCall;
procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean); export; stdCall;

// internal functions only
procedure COM_CreatePort;
procedure COM_FreePort;
procedure COM_Open;
procedure COM_Close;
function  ByteToBits(b: byte; nBits: byte): string;
procedure Delay(ms: integer);
function  COM_ReadAllValues: boolean;
function  COM_WriteAllValues: boolean;
function  TLC1543_ReadValues(var AInAr: array of double): boolean;
function  TLC1543_WriteAddr(state: boolean): boolean;
function  TLC1543_WriteCLK(state: boolean): boolean;
function  TLC1543_ReadDataBit: boolean;
function  TLC1543_ReadEOC: boolean;
procedure SetError(bState: boolean; sErr: shortString);


const
  MaxAnalogInputs = 14;    // 11 inputs + 3 test ref. values
  MaxDigitalInputs = 0;
  DataType_Byte = 0;
  DataType_Word = 1;
  DataType_DWord = 2;

  // ports addresses
  //LPT1PortBaseAddr = $378;        // 888d
  //LPT2PortBaseAddr = $278;        // 632d
  COM1PortBaseAddr = $3F8;        // 1016d
  COM2PortBaseAddr = $2F8;        // 760d
  COM3PortBaseAddr = $3E8;        // 1000d
  COM4PortBaseAddr = $2E8;        // 744d
  //JOY1PortBaseAddr = $201;

var
  FHandle: integer; // to handle com port
  ComPort: TapdComPort;
  bPortCreated: boolean;
  bPortOpened: boolean;

  DataType: byte = DataType_DWord;

  LastErrorOccured: shortString;
  //LPTActivePortBaseAddr: word;
  COMActivePortBaseAddr: word;
  //JOYActivePortBaseAddr: word;
  KeepOriginalState: boolean;

  I_CTS, I_DSR, I_RI, I_DCD: boolean;
  O_DTR, O_RTS, O_TX: boolean;

resourceString
  sErrCOMRead = 'Erreur de lecture sur [COM]';


implementation

uses
  Windows,
  dialogs,  rmMisc,
  SysUtils,
  rmPorts,
  ZLPortIO;

procedure COM_CreatePort;
begin
  try
    ComPort := TapdComPort.create(application);
    ComPort.Open := false;
    ComPort.AutoOpen := false;
    ComPort.ComNumber := 1;
    bPortCreated := true;
  except
    bPortCreated := false;
  end;
end;

procedure COM_FreePort;
begin
  showmessage('Start COM_FreePort');
  ComPort.free;
  showmessage('Stop COM_FreePort');
end;

procedure COM_Open;
var
  sPort: String;
begin
  if not assigned(ComPort) then
  begin
    showmessage('COMPORT not Assigned !');
    exit;
  end;

  // close port before change params
  if ComPort.Open then
    COM_Close;

  // set port number
  case COMActivePortBaseAddr of
    COM1PortBaseAddr: ComPort.ComNumber := 1;
    COM2PortBaseAddr: ComPort.ComNumber := 2;
    COM3PortBaseAddr: ComPort.ComNumber := 3;
    COM4PortBaseAddr: ComPort.ComNumber := 4;
    else ComPort.ComNumber := 1;
  end;

  // open port
  try
    ComPort.open := true;
    bPortOpened := true;
  except
    bPortOpened := false;
  end;

  showmessage('Ouverture port COM : ' + boolToStr(bPortOpened, 'OK', 'Erreur !'));

end;

procedure COM_Close;
begin
  ComPort.Open := false;
end;

function ByteToBits(b: byte; nBits: byte): string;
var
  i: integer;
  bt: byte;
  s: string;
begin
  bt := $01;
  s := '';
  for i := 1 to nBits do begin
    if (b and bt) > 0 then
      s := '1' + s
    else
      s := '0' + s;
    {$R-}
    bt := bt shl 1;
    {$R+}
  end;
  result := s;
end;

procedure Delay(ms: integer);
var
  t0, t1: integer;
begin
  t0 := getTickCount;
  repeat
    t1 := getTickCount;
  until (t1 - t0) > ms;
end;

procedure GetPortsAvail(var LPT, COM, JOY: boolean);
begin
  LPT := false;
  COM := true;
  JOY := false;
end;

procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: word);
begin
  //LPTActivePortBaseAddr := LPTAddr;
  COMActivePortBaseAddr := COMAddr;
  //JOYActivePortBaseAddr := JOYAddr;
  COM_Close;
  COM_Open;
end;

{
************************************************************************
  serial port
************************************************************************
}

function COM_ReadAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
begin
  Addr := COMActivePortBaseAddr + 6;

  // read value on port if system IS NOT WinNT
  if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
    try
      Case DataType of
        DataType_Byte :
          asm
            mov edx,Addr
            xor eax,eax
            in al,dx
            mov Data,eax
          end;
        DataType_Word :
          asm
            mov edx,Addr
            xor eax,eax
            in ax,dx
            mov Data,eax
          end;
        DataType_DWord:
          asm
            mov edx,Addr
            xor eax,eax
            in eax,dx
            mov Data,eax
          end;
       end;
      {
      asm
        mov dx, Addr
        in al, dx
        mov Data, al
      end;
      }
    except
      result := false;
      exit;
    end;

  // read status value on port if system IS WinNT
  end else begin
    try
      Data := ZLIOPortRead(Addr, DataType);
    except
      result := false;
      exit;
    end;

  end;

  // extract bits value for individual status line
  I_DCD := ((Data and 128) = 128);
  I_RI := ((Data and 64) = 64);
  I_DSR := ((Data and 32) = 32);
  I_CTS := ((Data and 16) = 16);

  result := true;
end;

function COM_WriteAllValues: boolean;
var
  Data: DWord;
  Addr: DWord;
  i: byte;
begin
  result := true;

  for i := 1 to 2 do
  begin
    Data := 0;
    case i of

      // set Modem Commands address and Value
      1 :
        begin
          Addr := COMActivePortBaseAddr + 3;
          if O_TX then Data := Data + 64;    // TX = bit 6
        end;

      // set Line Commands address and Value
      2 :
        begin
          Addr := COMActivePortBaseAddr + 4;
          if O_DTR then Data := Data + 1;    // DTR = bit 0
          if O_RTS then Data := Data + 2;    // RTS = bit 1
        end;

    end;

    // write value on port if system IS NOT WinNT
    if Win32Platform <> VER_PLATFORM_WIN32_NT then begin
      try

        Case DataType of
          DataType_Byte :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, al
            end;
          DataType_Word :
            asm
              mov edx, Addr
              mov eax, Data
              out dx, ax
            end;
          DataType_DWord:
            asm
              mov edx, Addr
              mov eax, Data
              out dx, eax
            end;
        end;
        {
        asm
          mov dx, Addr
          mov ax, Data
          out dx, al
        end;
        }

      except
        result := false;
        break;
      end;

    // write value on port if system IS WinNT
    end else begin
      try
        ZLIOPortWrite(Addr, DataType, Data);
        //showmessage(format('I=%d, Addr=%d, Data=%d', [i, addr, Data]));
      except
        showmessage('OUPS !');
        result := false;
        break;
      end;

    end  // if then else

  end;  // for

end;

procedure GetShortDesc(var s: ShortString);
begin
  s := '11 Entres Anal. sur port COM';
end;

procedure GetLongDesc(var s: ShortString);
begin
  s := 'Interface 11 entres analogiques base sur le TLC1543';
end;

procedure GetCANFullScale(var d: double);
begin
  d := 5.0;
end;

procedure GetCANResolution(var i: byte);
begin
  i := 10;
end;

procedure GetAnalogInputsCapacity(var i: byte);
begin
  i := MaxAnalogInputs;
end;

procedure GetDigitalInputsCapacity(var i: byte);
begin
  i := MaxDigitalInputs;
end;

procedure GetInputsLabel(var ar: array of ShortString);
var
  i: integer;
  Min, Max: byte;
begin
  // labels for analog inputs
  Min := 0;
  GetAnalogInputsCapacity(Max);
  // analog inputs
  for i := Min to pred(Max - 3) do
    ar[i] := format('TLC1543 Entre anal. #%.3d', [i + 1]);
  // test values
  for i := (Min + 11) to pred(Max) do
    ar[i] := format('TLC1543 Test Rf. #%.3d', [i - 11 + 1]);
end;

procedure InterfaceDetected(var b: boolean);
begin
  // do the job
  b := true;
end;

procedure RunTest(var bResult: boolean);
var
  Old_DTR, Old_RTS, Old_TX: boolean;
  i: integer;
begin

  // save initial values
  Old_DTR := O_DTR;
  Old_RTS := O_RTS;
  Old_TX := O_TX;

  // run test
  O_DTR := false;
  O_RTS := false;
  O_TX := false;
  try
    for i:= 0 to 4 do
    begin
      O_DTR := (i = 1);
      O_RTS := (i = 2);
      O_TX := (i = 3);
      bResult := COM_WriteAllValues;
      delay(100);
    end;

    // restore initial values
    O_DTR := Old_DTR;
    O_RTS := Old_RTS;
    O_TX := Old_TX;
    bResult := COM_WriteAllValues;
  except
    bResult := false;
  end;
end;

procedure InitHard(var bResult: boolean);
begin
  O_DTR := true;
  O_RTS := true;
  O_TX := false;
  bResult := COM_WriteAllValues;
end;

procedure ResetHard(var bResult: boolean);
var
  t0, t1: integer;
begin
  InitHard(bResult);
  // send multiple edges on TX line
  t0 := GetTickCount;
  repeat
    O_TX := not O_TX;
    bResult := COM_WriteAllValues;
    if not bResult then
      exit;
    delay(50);
    t1 := GetTickCount;
  until (t1 - t0) > 1000;
  O_TX := false;
  bResult := COM_WriteAllValues;
  if not bResult then
    exit;
  bResult := TLC1543_WriteAddr(false);
  if not bResult then
    exit;
  bResult := TLC1543_WriteCLK(false);
  if not bResult then
    exit;
  //delay(500, true);
  bResult := true;
end;

procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean);
var
  ReadOK: boolean;
  AInAr: array[0..13] of double;
  i: integer;
begin
  try
    ReadOK := TLC1543_ReadValues(AInAr);
    for i := 0 to 13 do
      arA[i] := AInAr[i];
  except
    ReadOK := false;
  end;
  bResult := ReadOK;
  if not ReadOK then
    SetError(true, sErrCOMRead);
end;

procedure SetError(bState: boolean; sErr: shortString);
begin
  if bState then
    LastErrorOccured := sErr
  else
    LastErrorOccured := '';
end;

function  TLC1543_ReadValues(var AInAr: array of double): boolean;
var
  i, j, temp: integer;
  addr: string[4];
  val: integer;
  EOC: boolean;
  timeOut: integer;
  t0, t1: longInt;
  rVal: double;
begin
  timeOut := 20;

  // input 0..10 are analog inputs
  // input 11..13 are internal test values
  for i := 0 to 13 do begin

    // input to read addr
    addr := byteToBits(i, 4);

    // get TLC1543 sampled values
    temp := 1024;
    val := 0;
    for j := 1 to 10 do
    begin
      temp := temp div 2;
      if j < 5 then
        TLC1543_WriteAddr(copy(Addr, j, 1) = '1')  //  via RTS
      else
      TLC1543_WriteAddr(false);                    //  via RTS
      TLC1543_WriteCLK(true);       // pos clock edge, via DTR
      if TLC1543_ReadDataBit then
        val := val + temp;          //               , via CTS
      TLC1543_WriteCLK(false);      // neg clock edge, via DTR
    end;

    // wait for End Of Conversion
    t0 := GetTickCount;
    repeat
      EOC := TLC1543_ReadEOC;                       // via DSR
      t1 := GetTickCount;
    until EOC or ((t1 - t0) > timeOut);

    // at the moment, don't keep original value
    rVal := val / 10000;

    // put result in values array
    case i of
      0 : AInAr[13] := rVal;
      1..13 : AInAr[i - 1] := rVal;
    end;

  end;
end;

function  TLC1543_WriteAddr(state: boolean): boolean;
begin
  O_RTS := not state;
  result := COM_WriteAllValues;
end;

function  TLC1543_WriteCLK(state: boolean): boolean;
begin
  O_DTR := not state;
  result := COM_WriteAllValues;
end;

function TLC1543_ReadDataBit: boolean;
begin
  COM_ReadAllValues;
  result := not I_CTS;
end;

function TLC1543_ReadEOC: boolean;
begin
  COM_ReadAllValues;
  result := not I_DSR;
end;


initialization
  COM_CreatePort;
  COM_Open;


finalization
  COM_Close;
  COM_FreePort;


end.
