unit _Simulation_Main;

interface

// exported functions
procedure GetPortsAvail(var LPT, COM, JOY: boolean); export; stdCall;
procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: integer);  export; stdCall;
procedure GetShortDesc(var s: ShortString); export; stdCall;
procedure GetLongDesc(var s: ShortString); export; stdCall;
procedure GetCANFullScale(var d: double); export; stdCall;
procedure GetCANResolution(var i: byte); export; stdCall;
procedure GetAnalogInputsCapacity(var i: byte); export; stdCall;
procedure GetDigitalInputsCapacity(var i: byte); export; stdCall;
procedure GetInputsLabel(var ar: array of ShortString); export; stdCall;
procedure InterfaceDetected(var b: boolean); export; stdCall;
procedure RunTest(var bResult: boolean); export; stdCall;
procedure InitHard(var bResult: boolean); export; stdCall;
procedure ResetHard(var bResult: boolean); export; stdCall;
procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean); export; stdCall;

// internal functions only
function  GetRandomValue: double;
function  GetRandomState: boolean;
procedure SetError(bState: boolean; sErr: shortString);

const
  MaxAnalogInputs = 64;
  MaxDigitalInputs = 64;

var
  LastErrorOccured: shortString;
  LPTActivePortBaseAddr: integer;
  COMActivePortBaseAddr: integer;
  JOYActivePortBaseAddr: integer;
  KeepOriginalState: boolean;


implementation

uses
  SysUtils;

procedure GetPortsAvail(var LPT, COM, JOY: boolean);
begin
  LPT := false;
  COM := false;
  JOY := false;
end;

procedure SetActivePortBaseAddr(LPTAddr, COMAddr, JOYAddr: integer);
begin
  LPTActivePortBaseAddr := LPTAddr;
  COMActivePortBaseAddr := COMAddr;
  JOYActivePortBaseAddr := JOYAddr;
end;

function  GetRandomValue: double;
begin
  result := random(1000) / 1000;
end;

function  GetRandomState: boolean;
var
  i: integer;
begin
  i := random(2);
  result := boolean(i);
end;

procedure GetShortDesc(var s: ShortString);
begin
  s := '64 entres anal. / 64 entres num. simules';
end;

procedure GetLongDesc(var s: ShortString);
begin
  s := '64 entres analogique / 64 entres numriques simules';
end;

procedure GetCANFullScale(var d: double);
begin
  d := 0.0;
end;

procedure GetCANResolution(var i: byte);
begin
  i := 0;
end;

procedure GetAnalogInputsCapacity(var i: byte);
begin
  i := MaxAnalogInputs;
end;

procedure GetDigitalInputsCapacity(var i: byte);
begin
  i := MaxDigitalInputs;
end;

procedure GetInputsLabel(var ar: array of ShortString);
var
  i: integer;
  Min, Max: byte;
begin
  // labels for analog inputs
  Min := 0;
  GetAnalogInputsCapacity(Max);
  for i := Min to pred(Max) do
    ar[i] := format('Simul. Entre anal. #%.3d', [i + 1]);
  // labels for digital inputs
  Min := 64;
  GetDigitalInputsCapacity(Max);
  Max := Max + Min;
  for i := Min to pred(Max) do
    ar[i] := format('Simul. Entre num. #%.3d', [i - 64 + 1]);
end;

procedure InterfaceDetected(var b: boolean);
begin
  b := true;
end;

procedure RunTest(var bResult: boolean);
begin
  bResult := true;
end;

procedure InitHard(var bResult: boolean);
begin
  // do nothing
  bResult := true;
end;

procedure ResetHard(var bResult: boolean); 
begin
  // do nothing
  bResult := true;
end;

procedure ReadValues(var arA: array of double;
                     var arD: array of boolean;
                     var bResult: boolean);
var
  i: integer;
begin
   for i := 0 to 63 do
    arA[i] := GetRandomValue;
   for i := 0 to 63 do
    arD[i] := GetRandomState;
  bResult := true;
end;

procedure SetError(bState: boolean; sErr: shortString);
begin
  if bState then
    LastErrorOccured := sErr
  else
    LastErrorOccured := '';
end;


end.
