unit frmPlugins;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls;

type
  TpipGetPortsAvail = procedure(var LPT, COM, JOY: boolean); stdcall;
  TpipSetActivePortBaseAddr = procedure(LPT, COM, JOY: word); stdcall;
  TpipGetShortDesc = procedure(var s: shortString); stdcall;
  TpipGetLongDesc = procedure(var s: shortString); stdcall;
  TpipGetCANResolution = procedure(var i: byte); stdcall;
  TpipGetCANFullScale = procedure(var d: double); stdcall;
  TpipGetAnalogInputsCapacity = procedure(var i: byte); stdcall;
  TpipGetDigitalInputsCapacity = procedure(var i: byte); stdcall;
  TpipGetInputsLabel = procedure (var ar: array of ShortString); stdcall;
  TpipInterfaceDetected = procedure(var b: boolean); stdCall;
  TpipRunTest = procedure(var b: boolean); stdCall;
  TpipInitHard = procedure(var b: boolean); stdCall;
  TpipResetHard = procedure(var b: boolean); stdCall;
  TpipReadValues = procedure(var arA: array of double;
                             var arD: array of boolean;
                             var b: boolean); stdCall;

  TfmPlugins = class(TForm)
    lvPlugins: TListView;
  private
    procedure Plugin_AddToList;
    function  Plugin_InitHard: boolean;
    function  Plugin_GetInputsLabel: boolean;
    { Dclarations prives }
  public
    { Dclarations publiques }
    procedure Plugin_InitParams;
    procedure Plugin_GetAllPluginsDesc;
    function  Plugin_Load(const sFile: string): boolean;
    procedure Plugin_Unload(bInitParams: boolean);
    function  Plugin_CheckForLoaded: boolean;
    procedure Plugin_GetParams;
    procedure Plugin_SetActive(sPluginFileName: string);
    function  Plugin_PluginNameToIndex(s: string): integer;
    function  Plugin_SetActivePortBaseAddr: boolean;
  end;

const
  cPlugin_Mask = '*.hwl';
  cPipGetPortsAvail = 'GetPortsAvail';
  cPipSetActivePortBaseAddr = 'SetActivePortBaseAddr';
  cPipGetShortDesc = 'GetShortDesc';
  cPipGetLongDesc = 'GetLongDesc';
  cPipGetCANResolution = 'GetCANResolution';
  cPipGetCANFullScale = 'GetCANFullScale';
  cPipGetAnalogInputsCapacity = 'GetAnalogInputsCapacity';
  cPipGetDigitalInputsCapacity = 'GetDigitalInputsCapacity';
  cPipGetInputsLabel = 'GetInputsLabel';
  cPipInterfaceDetected = 'InterfaceDetected';
  cPipRunTest = 'RunTest';
  cPipInitHard = 'InitHard';
  cPipResetHard = 'ResetHard';
  cPipReadValues = 'ReadValues';

var
  fmPlugins: TfmPlugins;

  // DLL procedures
  pipSetActivePortBaseAddr: TpipSetActivePortBaseAddr;
  pipInterfaceDetected: TPipInterfaceDetected;
  pipGetInputsLabel: TPipGetInputsLabel;
  pipRunTest: TPipRunTest;
  pipInitHard: TpipInitHard;
  pipResetHard: TpipResetHard;
  pipReadValues: TPipReadValues;


implementation

uses
  resStr,
  rmMisc,
  uChartSeries,
  frmMain;

{$R *.DFM}


procedure TfmPlugins.Plugin_GetAllPluginsDesc;
var
  sr: TSearchRec;
  found: integer;
begin
  lvPlugIns.items.Clear;
  try
    found := FindFirst(AppPath + cPlugin_Mask, 0, sr);
    while found = 0 do
    begin

      // load plugin
      Plugin_Load(sr.name);

      // get plugin params
      Plugin_GetParams;

      // add plugin params to plugins list
      Plugin_AddToList;

      // unload plugin
      Plugin_Unload(true);

      // continue search
      found := FindNext(sr);

    end;

    // init plugin params
    Plugin_InitParams;

  finally
    FindClose(sr);
  end;
end;

procedure TfmPlugins.Plugin_InitParams;
begin
  Plugin.PluginFileName := '';
  Plugin.PluginLoaded := false;
  Plugin.ShortDesc := '???';
  Plugin.LongDesc := '???';
  Plugin.CANResolution := 0;
  Plugin.CANFullScale := 0.0;
  Plugin.AnalogInputCapacity := 0;
  Plugin.DigitalInputCapacity := 0;
  Plugin.LPTCanBeUsed := false;
  Plugin.COMCanBeUsed := false;
  Plugin.JOYCanBeUsed := false;
end;

function TfmPlugins.Plugin_Load(const sFile: string): boolean;
var
  sPluginFileName: string;
  bResult: boolean;
begin
  result := false;
  //showmessage('Load plugin : ' + sFile);

  // file specified ?
  if extractFileName(sFile) = '' then
    exit;

  // retrieve full path of plugin
  sPluginFileName := AppPath + extractFileName(sFile);

  // specified plugin exists ?
  if not fileExists(sPluginFileName) then begin
    showMessage(format(sErrPluginFileNotExists, [sPluginFileName]));
    exit;
  end;

  // if plugin already loaded, unload it before loading new plugin
  // at the same time, init plugin params
  if Plugin.PluginLoaded then
    Plugin_Unload(true);

  try
    // load DLL
    Plugin.PluginHandle := loadLibrary(PChar(sPluginFileName));
    if Plugin.PluginHandle <> 0 then
    begin
      Plugin.PluginFileName := sPluginFileName;
      Plugin.PluginLoaded := true;
      fmMain.StatusBar_RefreshHWInterfaceAndPort;
    end;
  except
    exit;
  end;
  result := true;
end;

procedure TfmPlugins.Plugin_Unload(bInitParams: boolean);
begin
  if Plugin.PluginLoaded then
  begin
    try
      // unload actualy used plugin from memory
      FreeLibrary(Plugin.PluginHandle);
    except
    end;
  end;
  plugin.PluginLoaded := false;
  fmMain.StatusBar_RefreshHWInterfaceAndPort;
  if bInitParams then
    Plugin_InitParams;
end;

function TfmPlugins.Plugin_CheckForLoaded: boolean;
begin
  result := plugin.PluginLoaded;
  if not plugin.PluginLoaded then begin
    showMessage(sNoPluginLoaded);
    abort;
  end;
end;

procedure TfmPlugins.Plugin_GetParams;
var
  s: shortString;
  i: byte;
  d: double;
  bCOM, bLPT, bJOY: boolean;
  // pip = PlugInProc
  pipGetPortsAvail: TPipGetPortsAvail;
  pipGetShortDesc: TpipGetShortDesc;
  pipGetLongDesc: TpipGetLongDesc;
  pipGetCANResolution: TpipGetCANResolution;
  pipGetCANFullScale: TPipGetCANFullScale;
  pipGetAnalogInputsCapacity: TpipGetAnalogInputsCapacity;
  pipGetDigitalInputsCapacity: TpipGetDigitalInputsCapacity;
begin
  try

    // get ports (COM, LPT and JOY) that can be used
    pipGetPortsAvail := GetProcAddress(Plugin.PluginHandle, cPipGetPortsAvail);
    boolToStr(assigned(pipGetPortsAvail), 'OK', 'Error');
    if assigned(pipGetPortsAvail) then begin
      pipGetPortsAvail(bLPT, bCOM, bJOY);
      Plugin.LPTCanBeUsed := bLPT;
      Plugin.COMCanBeUsed := bCOM;
      Plugin.JOYCanBeUsed := bJOY;
    end;

    // set active ports to use
    pipSetActivePortBaseAddr := GetProcAddress(Plugin.PluginHandle, cPipSetActivePortBaseAddr);
    boolToStr(assigned(pipSetActivePortBaseAddr), 'OK', 'Error');

    // get short desc
    pipGetShortDesc := GetProcAddress(Plugin.PluginHandle, cPipGetShortDesc);
    boolToStr(assigned(pipGetShortDesc), 'OK', 'Error');
    if assigned(pipGetShortDesc) then begin
      pipGetShortDesc(s);
      Plugin.ShortDesc := s;
    end;

    // get long desc
    pipGetLongDesc := GetProcAddress(Plugin.PluginHandle, cPipGetLongDesc);
    boolToStr(assigned(pipGetLongDesc), 'OK', 'Error');
    if assigned(pipGetLongDesc) then begin
      pipGetLongDesc(s);
      Plugin.LongDesc := s;
    end;

    // get CAN resolution
    pipGetCANResolution := GetProcAddress(Plugin.PluginHandle, cPipGetCANResolution);
    boolToStr(assigned(pipGetCANResolution), 'OK', 'Error');
    if assigned(pipGetCANResolution) then begin
      pipGetCANResolution(i);
      Plugin.CANResolution := i;
    end else begin
      Plugin.CANResolution := 0;
    end;

    // get CAN full scale value (for example, 5.0V for TLC1543)
    pipGetCANFullScale := GetProcAddress(Plugin.PluginHandle, cPipGetCANFullScale);
    boolToStr(assigned(pipGetCANFullScale), 'OK', 'Error');
    if assigned(pipGetCANFullScale) then begin
      pipGetCANFullScale(d);
      Plugin.CANFullScale := d;
    end else begin
      Plugin.CANFullScale := 0.0;
    end;

    // get number of analog inputs
    pipGetAnalogInputsCapacity := GetProcAddress(Plugin.PluginHandle, cPipGetAnalogInputsCapacity);
    boolToStr(assigned(pipGetAnalogInputsCapacity), 'OK', 'Error');
    if assigned(pipGetAnalogInputsCapacity) then begin
      pipGetAnalogInputsCapacity(i);
      Plugin.AnalogInputCapacity := i;
    end else begin
      Plugin.AnalogInputCapacity := 0;
    end;

    // get number of digital inputs
    pipGetDigitalInputsCapacity := GetProcAddress(Plugin.PluginHandle, cPipGetDigitalInputsCapacity);
    boolToStr(assigned(pipGetDigitalInputsCapacity), 'OK', 'Error');
    if assigned(pipGetDigitalInputsCapacity) then begin
      pipGetDigitalInputsCapacity(i);
      Plugin.DigitalInputCapacity := i;
    end else begin
      Plugin.DigitalInputCapacity := 0;
    end;

    // get Inputs Label proc address
    pipGetInputsLabel := GetProcAddress(Plugin.PluginHandle, cPipGetInputsLabel);
    boolToStr(assigned(pipGetInputsLabel), 'OK', 'Error');

    // get InterfaceDetected proc address
    pipInterfaceDetected := GetProcAddress(Plugin.PluginHandle, cPipInterfaceDetected);
    boolToStr(assigned(pipInterfaceDetected), 'OK', 'Error');

    // get runTest proc address
    pipRunTest := GetProcAddress(Plugin.PluginHandle, cPipRunTest);
    boolToStr(assigned(pipRunTest), 'OK', 'Error');

    // get Init Hard proc address
    pipInitHard := GetProcAddress(Plugin.PluginHandle, cPipInitHard);
    boolToStr(assigned(pipInitHard), 'OK', 'Error');

    // get Reset Hard proc address
    pipResetHard := GetProcAddress(Plugin.PluginHandle, cPipResetHard);
    boolToStr(assigned(pipResetHard), 'OK', 'Error');

    // get ReadValues proc address
    pipReadValues := GetProcAddress(Plugin.PluginHandle, cPipReadValues);
    boolToStr(assigned(pipReadValues), 'OK', 'Error');

  except
    // init plugin params
    Plugin_InitParams;
  end;
end;

procedure TfmPlugins.Plugin_AddToList;
var
  NewItem: TListItem;
  s: string;
begin
  if not Plugin.PluginLoaded then
    exit;

  //showmessage('Add Plugin to list : ' + sFile);
  NewItem := lvPlugins.items.Add;
  try

    // print plugin file name
    NewItem.Caption := upperCase(extractFileName(Plugin.PluginFileName));

    // print short desc
    NewItem.SubItems.Add(Plugin.ShortDesc);

    // print long desc
    NewItem.SubItems.Add(Plugin.LongDesc);

    // print analog inputs capacity
    NewItem.SubItems.Add(intToStr(Plugin.AnalogInputCapacity));

    // print digital inputs capacity
    NewItem.SubItems.Add(intToStr(Plugin.DigitalInputCapacity));

    // print CAN resolution
    s := boolToStr(Plugin.CANResolution > 0, format('%d bits', [Plugin.CANResolution]), '-');
    NewItem.SubItems.Add(s);

    // print CAN Full scale
    s := boolToStr(Plugin.CANFullScale > 0.0, format('%f V', [Plugin.CANFullScale]), '-');
    NewItem.SubItems.Add(s);

  except
  end;
end;

procedure TfmPlugins.Plugin_SetActive(sPluginFileName: string);
var
  s, sPluginName: string;
  sep: integer;
  bResult: boolean;
begin

  // get plugin name (accept comment after name if separated by '-' char
  s := upperCase(sPluginFileName);
  sep := pos('HWL - ', s);

  // remove comment and keep plugin file name
  if sep > 0 then
    sPluginName := copy(s, 1, sep + 2)

  // keep as this
  else
    sPluginName := extractFileName(sPluginFileName);

  // load plugin
  if Plugin_Load(appPath + sPluginName) then begin
    // get plugin params
    Plugin_GetParams;
    Plugin_SetActivePortBaseAddr;
    Plugin_InitHard;
    Plugin_GetInputsLabel;
    Chart_SetSeriesActive;
  end
  else
  begin
    showmessage(sErrPluginLoadFail);
  end;

  fmMain.StatusBar_RefreshHWInterfaceAndPort;
end;

function TfmPlugins.Plugin_PluginNameToIndex(s: string): integer;
var
  item: TListItem;
begin
  item := lvPlugins.FindCaption(0, s, false, true, true);
  if item <> nil then
    result := item.Index
  else
    result := -1;
end;

function TfmPlugins.Plugin_SetActivePortBaseAddr: boolean;
begin
  // is a plugin loaded ? if not, abort.
  fmPlugins.Plugin_CheckForLoaded;

  try
    if assigned(pipSetActivePortBaseAddr) then begin
      pipSetActivePortBaseAddr(
        userPrefs.PortLPTToUse,
        userPrefs.PortCOMToUse,
        userPrefs.PortJOYToUse);
      result := true;
    end
    else
      result := false;
  except
    result := false;
  end;
end;

function TfmPlugins.Plugin_InitHard: boolean;
var
  bResult: boolean;
begin
  // is a plugin loaded ? if not, abort.
  fmPlugins.Plugin_CheckForLoaded;

  try
    if assigned(pipInitHard) then begin
      pipInitHard(bResult);
      result := bResult;
    end
    else
      result := false;
  except
    result := false;
  end;
end;

function TfmPlugins.Plugin_GetInputsLabel: boolean;
begin
  // is a plugin loaded ? if not, abort.
  fmPlugins.Plugin_CheckForLoaded;

  // read labels
  fmMain.ActPluginGetInputsLabelExecute(nil);

end;


end.
