unit frmReport01;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, QuickRpt, Qrctrls;

type
  TfmReport01 = class(TForm)
    QuickRep1: TQuickRep;
    DetailBand1: TQRBand;
    lblEvent: TQRLabel;
    TitleBand1: TQRBand;
    PageHeaderBand1: TQRBand;
    lblMainTitle: TQRLabel;
    QRSysData1: TQRSysData;
    lblButtonIndex: TQRLabel;
    lblDateTime: TQRLabel;
    lblDuration: TQRLabel;
    PageFooterBand1: TQRBand;
    QRSysData2: TQRSysData;
    ColumnHeaderBand1: TQRBand;
    lblColumn0: TQRLabel;
    lblColumn1: TQRLabel;
    lblColumn2: TQRLabel;
    lblColumn3: TQRLabel;
    QRLabel6: TQRLabel;
    lblEventsCount: TQRLabel;
    QRLabel9: TQRLabel;
    lblLastEventDateTime: TQRLabel;
    QRLabel7: TQRLabel;
    lblPriorEventDateTime: TQRLabel;
    lblSubTitle: TQRLabel;
    procedure QuickRep1BeforePrint(Sender: TCustomQuickRep;
      var PrintReport: Boolean);
    procedure QuickRep1NeedData(Sender: TObject; var MoreData: Boolean);
    procedure QuickRep1StartPage(Sender: TCustomQuickRep);
    procedure FormCreate(Sender: TObject);
    procedure DetailBand1BeforePrint(Sender: TQRCustomBand;
      var PrintBand: Boolean);
  private
    { Dclarations prives }
    CurrentIndex: integer;
    CurrentTextColor: TColor;
    CurrentBGColor: TColor;
    sMainTitle: string;
    sSubTitle: string;
  public
    { Dclarations publiques }
  end;

var
  fmReport01: TfmReport01;

implementation

uses
  uResStr, uUserPrefs, uLog,
  frmMain, frmPrintOptions;

{$R *.DFM}

procedure TfmReport01.FormCreate(Sender: TObject);
begin

  // main title and sub-title
  if UserPrefs.ReportMainTitle <> '' then
    sMainTitle := UserPrefs.ReportMainTitle
  else
    sMainTitle := UserPrefs.AppMainTitle;

  if UserPrefs.ReportSubTitle <> '' then
    sSubTitle := UserPrefs.ReportSubTitle
  else
    sSubTitle := sReportSubTitleDefault;
  lblSubTitle.caption := sSubTitle;

  // grid headers (columns caption)
  lblColumn0.Caption := userPrefs.GridColumnCaption0;
  lblColumn1.Caption := userPrefs.GridColumnCaption1;
  lblColumn2.Caption := userPrefs.GridColumnCaption2;
  lblColumn3.Caption := userPrefs.GridColumnCaption3;

end;

procedure TfmReport01.QuickRep1StartPage(Sender: TCustomQuickRep);
begin

  if QuickRep1.PageNumber = 1 then
    lblMainTitle.caption := sMainTitle
  else
    lblMainTitle.caption := sMainTitle + ' (suite)'
end;

procedure TfmReport01.QuickRep1BeforePrint(Sender: TCustomQuickRep;
  var PrintReport: Boolean);
begin
  // header
  lblEventsCount.Caption := intToStr(fmMain.lvEventsList.Items.count);
  lblLastEventDateTime.Caption := fmMain.lvEventsList.Items[pred(fmMain.lvEventsList.Items.count)].SubItems[1];
  lblPriorEventDateTime.Caption := fmMain.lvEventsList.Items[0].SubItems[1];

  // prepart events printing
  Case UserPrefs.ReportSorting of
    0 : CurrentIndex := 0;
    1 : CurrentIndex := pred(fmMain.lvEventsList.Items.count);
  end;
end;

procedure TfmReport01.QuickRep1NeedData(Sender: TObject;
  var MoreData: Boolean);
var
  iColor: integer;
  bActive,
  bInvert: boolean;
  iMinDuration: integer;
  sLabel: string;
begin
  with fmMain.lvEventsList do
  begin

    // sort by increment or decrement ?
    Case UserPrefs.ReportSorting of
      0 : MoreData := (CurrentIndex < Items.count);
      1 : MoreData := (CurrentIndex > -1);
    end;

    // add one line
    if MoreData then
    begin

      // get text
      lblEvent.Caption := Items[CurrentIndex].Caption;
      lblButtonIndex.Caption := Items[CurrentIndex].SubItems[0];
      lblDateTime.Caption := Items[CurrentIndex].SubItems[1];
      lblDuration.Caption := Items[CurrentIndex].SubItems[2];

      // use colored events ?
      if UserPrefs.LogPrintColors then
      begin
        try
          iColor := strToInt(Items[CurrentIndex].SubItems[0]);
          Log_GetEventParams(
            UserPrefs,
            iColor,
            bActive,
            bInvert,
            iMinDuration,
            sLabel,
            CurrentTextColor,
            CurrentBGColor);
        except
          CurrentTextColor := clBlack;
          CurrentBGColor := clWindow;
        end;
        // text color
        lblEvent.Font.Color := CurrentTextColor;
        lblButtonIndex.Font.Color := CurrentTextColor;
        lblDateTime.Font.Color := CurrentTextColor;
        lblDuration.Font.Color := CurrentTextColor;
        // background color
        DetailBand1.Color := CurrentBGColor;
        lblEvent.Color := CurrentBGColor;
        lblButtonIndex.Color := CurrentBGColor;
        lblDateTime.Color := CurrentBGColor;
        lblDuration.Color := CurrentBGColor;
        end;
      end;

    Case UserPrefs.ReportSorting of
      0 : inc(CurrentIndex);
      1 : dec(CurrentIndex);
    end;
  end;
end;


procedure TfmReport01.DetailBand1BeforePrint(Sender: TQRCustomBand;
  var PrintBand: Boolean);
begin
  {
  try
    PrintBand :=
      (fmPrintOptions.rbLogPrintAll.Checked or
      (fmPrintOptions.rbLogPrintSelection.Checked and
      fmMain.lvEventsList.items[pred(CurrentIndex)].Selected));
  except
    //PrintBand := true;
    PrintBand := false;
  end;
  }
end;

end.
