unit uCapture;

interface

uses
  Forms, Classes, Windows, SysUtils, Graphics, Dialogs;

type

  TCaptureInput = record
    Active: boolean;
    Invert: boolean;
    OldState: boolean;
    NewState: boolean;
    StartTime: integer;
    index: integer;
    MinDuration: integer;
  end;

function  Capture_GetPortBaseAddress: integer;  overload;
function  Capture_GetPortBaseAddress(var LPTAddr, COMAddr, JOYAddr: integer): integer;  overload;
function  Capture_GetPortBaseAddressStr: string;
function  Capture_ComPortOpen: boolean;
procedure Capture_SetInput(
            var CaptureInput: TCaptureInput;
            iIndex: integer;
            bActive: boolean;
            bInvert: boolean;
            iStartTime: integer;
            iMinDuration: integer);
function  Capture_InitSystem: boolean;
function  Capture_TestSystem: boolean;
procedure Capture_Start;
procedure Capture_Stop;
procedure Capture_CheckForDayIsNew;
procedure Capture_AddEvent(
            bInsertState: boolean;
            bState: boolean;
            idx: integer;
            iDuration: integer);
procedure Capture_Simulate;
procedure Capture_ReadInputs;
procedure Capture_InterpretateInputState(
            var ci: TCaptureInput;
            bState: boolean);

var
  Capture_TestSystemOK: boolean;
  bCanCapture: boolean;
  Input0: TCaptureInput;
  Input1: TCaptureInput;
  Input2: TCaptureInput;
  Input3: TCaptureInput;
  Input4: TCaptureInput;
  Input5: TCaptureInput;
  Input6: TCaptureInput;
  Input7: TCaptureInput;
  Input8: TCaptureInput;
  Input9: TCaptureInput;
  Input10: TCaptureInput;
  Input11: TCaptureInput;
  Input12: TCaptureInput;
  {
  Input13: TCaptureInput;
  Input14: TCaptureInput;
  Input15: TCaptureInput;
  Input16: TCaptureInput;
  }

const
  COMIndexMin = 0;
  COMIndexMax = 3;
  LPTIndexMin = 4;
  LPTIndexMax = 5;
  JOYIndexMin = 6;
  JOYIndexMax = 6;
  JOYReadDigitalInputTimeOut = 1000;
  iMaxDuration = 5000;   // ms

implementation

uses
  rmPorts, rmMisc,
  uResStr,  uLog, uUserPrefs,
  frmMain;


function Capture_GetPortBaseAddress: integer;
begin
  result := -1;
  case UserPrefs.PortBaseAddressIndex of
    COMIndexMin + 0 : result := rmPorts.COM1PortBaseAddr;
    COMIndexMin + 1 : result := rmPorts.COM2PortBaseAddr;
    COMIndexMin + 2 : result := rmPorts.COM3PortBaseAddr;
    COMIndexMin + 3 : result := rmPorts.COM4PortBaseAddr;
    LPTIndexMin + 0 : result := rmPorts.LPT1PortBaseAddr;
    LPTIndexMin + 1 : result := rmPorts.LPT2PortBaseAddr;
    JOYIndexMin + 0 : result := rmPorts.JoystickPortBaseAddr;
  end;
end;

function  Capture_GetPortBaseAddress(var LPTAddr, COMAddr, JOYAddr: integer): integer;
begin
  result := -1;
  LPTAddr := -1;
  COMAddr := -1;
  JOYAddr := -1;
  case UserPrefs.PortBaseAddressIndex of
    COMIndexMin + 0 : result := rmPorts.COM1PortBaseAddr;
    COMIndexMin + 1 : result := rmPorts.COM2PortBaseAddr;
    COMIndexMin + 2 : result := rmPorts.COM3PortBaseAddr;
    COMIndexMin + 3 : result := rmPorts.COM4PortBaseAddr;
    LPTIndexMin + 0 : result := rmPorts.LPT1PortBaseAddr;
    LPTIndexMin + 1 : result := rmPorts.LPT2PortBaseAddr;
    JOYIndexMin + 0 : result := rmPorts.JoystickPortBaseAddr;
  end;
  case UserPrefs.PortBaseAddressIndex of
    COMIndexMin..COMIndexMax : COMAddr := result;
    LPTIndexMin..LPTIndexMax : LPTAddr := result;
    JOYIndexMin..JOYIndexMax : JOYAddr := result;
  end;
end;

function  Capture_GetPortBaseAddressStr: string;
begin
  result := '';
  case UserPrefs.PortBaseAddressIndex of
    COMIndexMin + 0 : result := 'COM1';
    COMIndexMin + 1 : result := 'COM2';
    COMIndexMin + 2 : result := 'COM3';
    COMIndexMin + 3 : result := 'COM4';
    LPTIndexMin + 0 : result := 'LPT1';
    LPTIndexMin + 1 : result := 'LPT2';
    JOYIndexMin + 0 : result := 'JOY1';
  end;
end;

function Capture_ComPortOpen: boolean;
begin
  try
    // close port
    fmMain.ApdComPort1.open := false;
    // change com number
    fmMain.ApdComPort1.ComNumber := UserPrefs.PortBaseAddressIndex + 1;
    // open port
    fmMain.ApdComPort1.open := true;
    result := true;
  except
    result := false;
  end;
end;

procedure Capture_SetInput(
            var CaptureInput: TCaptureInput;
            iIndex: integer;
            bActive: boolean;
            bInvert: boolean;
            iStartTime: integer;
            iMinDuration: integer);
begin
  with CaptureInput do
  begin
    index := iIndex;
    Active := bActive;
    Invert := bInvert;
    StartTime := iStartTime;
    MinDuration := iMinDuration;
  end;
end;

function  Capture_InitSystem: boolean;
var
  LPTAddr, COMAddr, JOYAddr: integer;
  iLPTMode: byte;
begin
  // assign correct index and reset start time to all input
  Capture_SetInput(input0, 0, userPrefs.LogEvent0Active, userPrefs.LogEvent0Invert, 0, UserPrefs.LogEvent0MinDuration);
  Capture_SetInput(input1, 1, userPrefs.LogEvent1Active, userPrefs.LogEvent1Invert, 0, UserPrefs.LogEvent1MinDuration);
  Capture_SetInput(input2, 2, userPrefs.LogEvent2Active, userPrefs.LogEvent2Invert, 0, UserPrefs.LogEvent2MinDuration);
  Capture_SetInput(input3, 3, userPrefs.LogEvent3Active, userPrefs.LogEvent3Invert, 0, UserPrefs.LogEvent3MinDuration);
  Capture_SetInput(input4, 4, userPrefs.LogEvent4Active, userPrefs.LogEvent4Invert, 0, UserPrefs.LogEvent4MinDuration);
  Capture_SetInput(input5, 5, userPrefs.LogEvent5Active, userPrefs.LogEvent5Invert, 0, UserPrefs.LogEvent5MinDuration);
  Capture_SetInput(input6, 6, userPrefs.LogEvent6Active, userPrefs.LogEvent6Invert, 0, UserPrefs.LogEvent6MinDuration);
  Capture_SetInput(input7, 7, userPrefs.LogEvent7Active, userPrefs.LogEvent7Invert, 0, UserPrefs.LogEvent7MinDuration);
  Capture_SetInput(input8, 8, userPrefs.LogEvent8Active, userPrefs.LogEvent8Invert, 0, UserPrefs.LogEvent8MinDuration);
  Capture_SetInput(input9, 9, userPrefs.LogEvent9Active, userPrefs.LogEvent9Invert, 0, UserPrefs.LogEvent9MinDuration);
  Capture_SetInput(input10, 10, userPrefs.LogEvent10Active, userPrefs.LogEvent10Invert, 0, UserPrefs.LogEvent10MinDuration);
  Capture_SetInput(input11, 11, userPrefs.LogEvent11Active, userPrefs.LogEvent11Invert, 0, UserPrefs.LogEvent11MinDuration);
  Capture_SetInput(input12, 12, userPrefs.LogEvent12Active, userPrefs.LogEvent12Invert, 0, UserPrefs.LogEvent12MinDuration);
  {
  Capture_SetInput(input13, 13, userPrefs.LogEvent13Active, userPrefs.LogEvent13Invert, 0, UserPrefs.LogEvent13MinDuration);
  Capture_SetInput(input14, 14, userPrefs.LogEvent14Active, userPrefs.LogEvent14Invert, 0, UserPrefs.LogEvent14MinDuration);
  Capture_SetInput(input15, 15, userPrefs.LogEvent15Active, userPrefs.LogEvent15Invert, 0, UserPrefs.LogEvent15MinDuration);
  Capture_SetInput(input16, 16, userPrefs.LogEvent16Active, userPrefs.LogEvent16Invert, 0, UserPrefs.LogEvent16MinDuration);
  }

  // assign correct addresses to ports
  Capture_GetPortBaseAddress(LPTAddr, COMAddr, JOYAddr);
  rmPorts.LPTActivePortBaseAddr := LPTAddr;
  rmPorts.COMActivePortBaseAddr := COMAddr;
  rmPorts.JOYActivePortBaseAddr := JOYAddr;

  // if LPT selected, set D0..D7 logical state to 0
  if UserPrefs.PortBaseAddressIndex in [LPTIndexMin..LPTIndexMax] then
  begin

    {
    rmPorts.ParallelPort_GetMode(LPTAddr, iLPTMode);
    showmessage('Befor, LPT mode = ' + inttostr(iLPTMode));
    rmPorts.ParallelPort_SetMode(LPTAddr, 1);
    rmPorts.ParallelPort_GetMode(LPTAddr, iLPTMode);
    showmessage('After, LPT mode = ' + inttostr(iLPTMode));
    }

    // write data value to verify reading is not made on data register
    rmPorts.ParallelPort_WriteDataValues(
      LPTAddr,
      false, false, false, false, false, false, false, false);

  end;

  // if com selected, open port
  if UserPrefs.PortBaseAddressIndex in [COMIndexMin..COMIndexMax] then
  begin
    Capture_ComPortOpen;
  end;

  result := true;
end;

function Capture_TestSystem: boolean;
var
  OSIsNT: boolean;
  NTLPTDriverOpened: boolean;
  LPTOpened, COMOpened, JOYOpened: boolean;
  LPT, COM, JOY: boolean;
begin
  COM := UserPrefs.PortBaseAddressIndex in [COMIndexMin..COMIndexMax];
  LPT := UserPrefs.PortBaseAddressIndex in [LPTIndexMin..LPTIndexMax];
  JOY := UserPrefs.PortBaseAddressIndex in [JOYIndexMin..JOYIndexMax];
  OSIsNT := fmMain.App_OSIsNT;
  NTLPTDriverOpened := rmPorts.LPT_zlIOStarted;
  COMOpened := fmMain.ApdComPort1.open;
  // JOYOpened := Game_Card_Detected;
  JOYOpened := false;

  Capture_TestSystemOK :=
    (COM and COMOpened) or
    (LPT and (not OSIsNT)) or
    (LPT and OSIsNT and NTLPTDriverOpened) or
    (JOY and JOYOpened);

  result := Capture_TestSystemOK;
end;

procedure Capture_Start;
begin
  EventsCountPartial := 0;

  // test if system is ready (if WinNT, is driver loaded ?)
  if not Capture_TestSystem then
  begin
    MessageDlg(sErrSystemNotReady, mtInformation, [mbOK], 0);
    exit;
  end;

  if not(UserPrefs.PortBaseAddressIndex in [LPTIndexMin..LPTIndexMax]) then
  begin
    MessageDlg('Not LPT !', mtInformation, [mbOK], 0);
    exit;
  end
  else
  begin
    //MessageDlg('Base Addr :' + inttostr(rmPorts.LPTActivePortBaseAddr), mtInformation, [mbOK], 0);
  end;

  // if auto save on new day, activate log (if not exists, create it)
  if UserPrefs.LogSaveAutoOnNewDay and
  (not FileExists(LogOpenedFileName)) then
  begin
    LogFileOfTheDay := fmMain.App_GetLogFileOfTheDay;
    Log_SetOpenedLogFile(LogFileOfTheDay);
  end;

  CaptureIsRunning := true;
  bCanCapture := true;
  fmMain.App_RefreshGUIMain;

  try
    // execute thread - Read data
    HThreadCapture := fmMain.BUThreadCapture.Execute(nil);
    bThreadIsRunning := true;

    // capture thread priority
    case UserPrefs.CaptureThreadPriority of
      0 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpIdle);
      1 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpLowest);
      2 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpLower);
      3 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpNormal);
      4 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpHigher);
      5 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpHighest);
      6 : fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpTimeCritical);
      else fmMain.BUThreadCapture.SetPriority(HThreadCapture, tpNormal);
    end;
  except
  end;

  // execute thread - save data
  {
  try
    if UserPrefs.LogSaveAutoOnNewDay then
    begin
      HThreadSaveData := fmMain.BUThreadSaveData.Execute(nil);
      // set save data thread to low priority (time not critical)
      fmMain.BUThreadSaveData.SetPriority(HThreadSaveData, tpLowest);
    end;
  except
  end;
  }

end;

procedure Capture_Stop;
const
  iMaxTime = 500;
var
  t0, t1: integer;
  bThStopped: boolean;
  bTimeEllapsed: boolean;
begin
  fmMain.ActCaptureStop.enabled := false;

  // stop thread - read data
  try
    fmMain.BUThreadCapture.QuitThread(HThreadCapture);

    // wait until thread really stop
    t0 := GetTickCount;
    repeat
      application.processMessages;
      t1 := GetTickCount;
      bThStopped := (not bThreadIsRunning);
      bTimeEllapsed := ((t1 - t0) > iMaxTime);
    until bThStopped or bTimeEllapsed;

  except
  end;

  // stop thread - save data
  {
  try
    if UserPrefs.LogSaveAutoOnNewDay then
      fmMain.BUThreadSaveData.QuitThread(HThreadSaveData);
  except
  end;
  }

  CaptureIsRunning := false;
  fmMain.App_RefreshGUIMain;
end;

procedure Capture_CheckForDayIsNew;
begin
  if not UserPrefs.LogSaveAutoOnNewDay then
    exit;

  // save to file if new day detected and if auto-save option is active
  if fmMain.App_DayIsNew then
  begin
    // disable capture during saving actually a limitation of this methode)
    bCanCapture := false;

    // save to file
    Log_SaveToFile(LogOpenedFileName);

    // set log file name with new day
    AppActualDay := now;
    LogFileOfTheDay := fmMain.App_GetLogFileOfTheDay;

    // load file of new day if existing, else create new one
    if FileExists(LogFileOfTheDay) then
      Log_LoadFromFile(LogFileOfTheDay)
    else
      Log_New(LogFileOfTheDay);

    // re-enable capture
    bCanCapture := true;
  end;
end;

procedure Capture_AddEvent(
            bInsertState: boolean;
            bState: boolean;
            idx: integer;
            iDuration: integer);
var
  EventItemTmp: TEventItem;
  bActive,
  bInvert: boolean;
  iMinDuration: integer;
  sLabel: string;
  clTextColor, clBGColor: TColor;
begin
  // inc counters
  inc(EventsCountPartial);
  inc(EventsCountTotal);

  // generate new properties for temp item
  Log_GetEventParams(
    UserPrefs,
    idx,
    bActive,
    bInvert,
    iMinDuration,
    sLabel,
    clTextColor,
    clBGColor);
  EventItemTmp.iIdx := idx;
  EventItemTmp.sEvent := sLabel;
  EventItemTmp.dtDateTime := now;
  EventItemTmp.iDuration := iDuration;

  // add temps item to list
  Log_AddLine(EventItemTmp, bInsertState, bState, true);

  // add item to log file
  if UserPrefs.LogSaveAutoOnNewEvent then
    Log_AddLineToFile(LogOpenedFileName);

  fmMain.App_RefreshGUIStatusBar;
  Log_SetOpenedLogModified(true);

end;

procedure Capture_Simulate;
var
  idx, duration: integer;
  t0, t1: integer;
begin
  idx := random(10000000);
  duration := random(1000) + 2;
  if idx in [0..12] then
  begin
    // event start
    case idx of
      0 : Capture_InterpretateInputState(Input0, true);
      1 : Capture_InterpretateInputState(Input1, true);
      2 : Capture_InterpretateInputState(Input2, true);
      3 : Capture_InterpretateInputState(Input3, true);
      4 : Capture_InterpretateInputState(Input4, true);
      5 : Capture_InterpretateInputState(Input5, true);
      6 : Capture_InterpretateInputState(Input6, true);
      7 : Capture_InterpretateInputState(Input7, true);
      8 : Capture_InterpretateInputState(Input8, true);
      9 : Capture_InterpretateInputState(Input9, true);
      10 : Capture_InterpretateInputState(Input10, true);
      11 : Capture_InterpretateInputState(Input11, true);
      12 : Capture_InterpretateInputState(Input12, true);
      {
      13 : Capture_InterpretateInputState(Input13, true);
      14 : Capture_InterpretateInputState(Input14, true);
      15 : Capture_InterpretateInputState(Input15, true);
      16 : Capture_InterpretateInputState(Input16, true);
      }
    end;

    // wait a few ms
    t0 := GetTickCount;
    repeat
      t1 := GetTickCount;
    until ((t1 - t0) > duration);

    // event start
    case idx of
      0 : Capture_InterpretateInputState(Input0, false);
      1 : Capture_InterpretateInputState(Input1, false);
      2 : Capture_InterpretateInputState(Input2, false);
      3 : Capture_InterpretateInputState(Input3, false);
      4 : Capture_InterpretateInputState(Input4, false);
      5 : Capture_InterpretateInputState(Input5, false);
      6 : Capture_InterpretateInputState(Input6, false);
      7 : Capture_InterpretateInputState(Input7, false);
      8 : Capture_InterpretateInputState(Input8, false);
      9 : Capture_InterpretateInputState(Input9, false);
      10 : Capture_InterpretateInputState(Input10, false);
      11 : Capture_InterpretateInputState(Input11, false);
      12 : Capture_InterpretateInputState(Input12, false);
      {
      13 : Capture_InterpretateInputState(Input13, false);
      14 : Capture_InterpretateInputState(Input14, false);
      15 : Capture_InterpretateInputState(Input15, false);
      16 : Capture_InterpretateInputState(Input16, false);
      }
    end;

  end;
end;

procedure Capture_ReadInputs;
var
  DoSimulation: boolean;
  iBaseAddr: word;
  I0, I1, I2, I3, I4, I5, I6, I7: boolean;
  I8, I9, I10, I11, I12: boolean;
  //I13, I14, I15, I16: boolean;
begin
  DoSimulation := false;

  // make capture simulation
  if DoSimulation then
  begin
    Capture_Simulate;
  end
  else

  // really read port inputs
  begin

    // read on LPT port
    if UserPrefs.PortBaseAddressIndex in [LPTIndexMin..LPTIndexMax] then
    begin

      // read LPT inputs
      iBaseAddr := rmPorts.LPTActivePortBaseAddr;
      rmPorts.ParallelPort_ReadAllValues(
        iBaseAddr,
        I8, I9, I10, I11, I12,
        //I13, I14, I15, I16,               // not used !!!
        I0, I1, I2, I3, I4, I5, I6, I7);

      // compare all actual input states with previous states
      Capture_InterpretateInputState(Input0, I0);
      Capture_InterpretateInputState(Input1, I1);
      Capture_InterpretateInputState(Input2, I2);
      Capture_InterpretateInputState(Input3, I3);
      Capture_InterpretateInputState(Input4, I4);
      Capture_InterpretateInputState(Input5, I5);
      Capture_InterpretateInputState(Input6, I6);
      Capture_InterpretateInputState(Input7, I7);
      Capture_InterpretateInputState(Input8, I8);
      Capture_InterpretateInputState(Input9, I9);
      Capture_InterpretateInputState(Input10, I10);
      Capture_InterpretateInputState(Input11, I11);
      Capture_InterpretateInputState(Input12, I12);
      {
      Capture_InterpretateInputState(Input13, I13);
      Capture_InterpretateInputState(Input14, I14);
      Capture_InterpretateInputState(Input15, I15);
      Capture_InterpretateInputState(Input16, I16);
      }
    end

    else

    // read on COM port
    if UserPrefs.PortBaseAddressIndex in [COMIndexMin..COMIndexMax] then
    begin
      // read COM inputs
      iBaseAddr := rmPorts.COMActivePortBaseAddr;
      rmPorts.SerialPort_ReadAllValues(
        iBaseAddr,
        I0, I1, I2, I3);
      // compare all actual input states with previous states
      Capture_InterpretateInputState(Input0, I0);
      Capture_InterpretateInputState(Input1, I1);
      Capture_InterpretateInputState(Input2, I2);
      Capture_InterpretateInputState(Input3, I3);
    end

    else

    // read on Joystic port
    if UserPrefs.PortBaseAddressIndex in [JOYIndexMin..JOYIndexMax] then
    begin
      // read digital joystick inputs
      iBaseAddr := rmPorts.JOYActivePortBaseAddr;
      rmPorts.JoystickPort_ReadDigitalValues(
        iBaseAddr,
        I0, I1, I2, I3,
        JOYReadDigitalInputTimeOut);
      // compare all actual input states with previous states
      Capture_InterpretateInputState(Input0, I0);
      Capture_InterpretateInputState(Input1, I1);
      Capture_InterpretateInputState(Input2, I2);
      Capture_InterpretateInputState(Input3, I3);
    end;

    // refresh input state indication (in main form status bar)
    {
    fmMain.StatusBar1.Panels[sbInputsStateIndex].text :=
      boolToStr(I0, '1', '0') + boolToStr(I1, '1', '0') +
      boolToStr(I2, '1', '0') + boolToStr(I3, '1', '0') +
      boolToStr(I4, '1', '0'){ + boolToStr(I6, '1', '0') +
      boolToStr(I7, '1', '0') + boolToStr(I8, '1', '0');
    }

  end;

end;

procedure Capture_InterpretateInputState(
            var ci: TCaptureInput;
            bState: boolean);
var
  EventItemTmp: TEventItem;
  iDuration: integer;
  idx: integer;
begin

  // input is active ?
  if not ci.Active then
    exit;

  // invert logic state ?
  if ci.Invert then
    ci.NewState := not bState
  else
    ci.NewState := bState;

  // get duration value
  if ci.StartTime > 0 then
    iDuration := GetTickCount - ci.StartTime
  else
    iDuration := 0;

  {
  // detect too long duration (more than 5s)
  if iDuration > iMaxDuration then
  begin
    EventItemTmp.iIdx := ci.index;
    EventItemTmp.sEvent := 'Timeout';
    EventItemTmp.dtDateTime := now;
    EventItemTmp.iDuration := iDuration;
    Log_AddLine(EventItemTmp, true, ci.newState, true);
    ci.StartTime := 0;
  end;
  }

  // input state change ?
  if ci.NewState <> ci.OldState then
  begin

    // suspend save data thread
    //fmMain.BUThreadSaveData.Suspend(HThreadSaveData);

    // if show edge change
    if UserPrefs.CaptureSaveEdge then
      Capture_AddEvent(true, ci.newState, ci.index, iDuration);

    // new event, so record start time
    if ci.NewState then
    begin
      ci.StartTime := GetTickCount;
    end
    else

    // return to initial state, so insert event
    begin
      ci.StartTime := 0;
      // if show duration
      if UserPrefs.CaptureSaveDuration and
      (not UserPrefs.CaptureSaveEdge)and
      (iDuration > ci.MinDuration) then
        Capture_AddEvent(false, ci.NewState, ci.index, iDuration);
    end;

    // assign new state to old state to allow compare at next capture
    ci.OldState := ci.NewState;

    // resume save data thread
    //fmMain.BUThreadSaveData.Resume(HThreadSaveData);

  end;

end;



end.
