unit uChartSeries;

interface

uses
  frmMain,
  SysUtils, Graphics, Dialogs, TeEngine, Series;


procedure Chart_InitSeries;
procedure Chart_SetSeriesActive;
function  Chart_GetSerieFixedColor(i: integer): TColor;
procedure Chart_InsertStartMarker;
//procedure Chart_FillWithNullValues(iMax: integer);
procedure Chart_AddXYAnalogPoint(idx, VPos: integer; XVal: TDateTime; YVal: double; bForce: boolean; YForceVal: double; bUseInput: boolean);
procedure Chart_AddXYDigitalPoint(idx, VPos: integer; XVal: TDateTime; YVal: double; bForce: boolean; YForceVal: double; bUseInput: boolean);
procedure Chart_RefreshSeries;
procedure Chart_Clear;
procedure Chart_Repaint;
procedure Chart_Print;
procedure Chart_CheckForSave;

implementation



procedure Chart_InitSeries;
var
  i: integer;
  NewSerie: TChartSeries;
begin
  for i := 0 to 127 do
  begin
    if fmMain.Chart1.Series[i] is TLineSeries then begin
      TLineSeries(fmMain.Chart1.Series[i]).title := inputs[i].caption;
      TLineSeries(fmMain.Chart1.Series[i]).active := inputs[i].inputEnabled;
      TLineSeries(fmMain.Chart1.Series[i]).colorEachPoint := false;
      TLineSeries(fmMain.Chart1.Series[i]).SeriesColor := inputs[i].LineColor;
      if i in [64..127] then  // only for digital inputs
        TLineSeries(fmMain.Chart1.Series[i]).stairs := true;
    end;
  end;
end;

{
procedure Chart_FillWithNullValues(iMax: integer);
var
  i, VPos, AInStart, AInStop, DInStart, DInStop, DebugCount: integer;
  InputHandled, InputIsAnalog, InputIsDigital: boolean;
  EnabledInputCount: integer;
  FirstPoint, iMaxReached: boolean;
begin
  DebugCount := 0;
  FirstPoint := true;

  // get valid interval
  fmMain.HWInterface_GetValidInputs(AInStart, AInStop, DInStart, DInStop);

  // fill with iMax points if not reached
  iMaxReached := (fmMain.chart1.SeriesList[0].Count >= iMax);

  // debug : only one loop
  iMaxReached := true;

  repeat

    inc(DebugCount);

    // count enabled inputs
    EnabledInputCount := 0;

    // add analog and digital values
    for i := 127 downTo 0 do
    begin

      // input disabled ?
      if not Inputs[i].InputEnabled then
        continue;

      // input handled by actual interface ?
      if not HWInterface_InputIsHandled(i, AInStart, AInStop, DInStart, DInStop) then
        continue;

      // count active inputs
      inc(EnabledInputCount);
      VPos := EnabledInputCount;


      // make a positive transition
      case i of
      000..063 : Chart_AddXYAnalogPoint(i, VPos, now, 0.0, true, 0.0, false);  // analog
      064..127 : Chart_AddXYDigitalPoint(i, VPos, now, 0.0, true, 0.0, false);  // digital
      end;
      case i of
      000..063 : Chart_AddXYAnalogPoint(i, VPos, now, 0.0, true, 1.0, false);  // analog
      064..127 : Chart_AddXYDigitalPoint(i, VPos, now, 0.0, true, 1.0, false);  // digital
      end;
      case i of
      000..063 : Chart_AddXYAnalogPoint(i, VPos, now, 0.0, true, 0.0, false);  // analog
      064..127 : Chart_AddXYDigitalPoint(i, VPos, now, 0.0, true, 0.0, false);  // digital
      end;

      continue;


      // draw new value or new state
      //showmessage(inttostr(fmMain.chart1.SeriesList[i].Count));
      // add points with specified values (not from inputs array)
      if FirstPoint then begin
        case i of
        000..063 : Chart_AddXYAnalogPoint(i, VPos, now, 0.0, true, 0.0, false);  // analog
        064..127 : Chart_AddXYDigitalPoint(i, VPos, now, 0.0, true, 0.0, false);  // digital
        end;
      end else begin
        case i of
          000..063 : Chart_AddXYAnalogPoint(i, VPos, now, 0.0, true, 0.0, false);  // analog
          064..127 : Chart_AddXYDigitalPoint(i, VPos, now, 0.0, true, 0.0, false);  // digital
        end;
      end;

    end;  // end for

    iMaxReached := true;



    iMaxReached := (fmMain.chart1.SeriesList[0].XValues.Count >= iMax);
    FirstPoint := false;

  until iMaxReached or (DebugCount > 2);

  Chart_Repaint;

end;
}

procedure Chart_SetSeriesActive;
var
  i: integer;
  AInStart, AInStop, DInStart, DInStop: integer;
  InputHandled, InputIsAcive, InputIsAnalog, InputIsDigital: boolean;
begin
  // change active inputs just after loading a new plugin

  // get valid interval
  fmMain.HWInterface_GetValidInputs(AInStart, AInStop, DInStart, DInStop);

  for i := 0 to 127 do
  begin
    // input handled by actual interface ?
    if not fmMain.HWInterface_InputIsHandled(i, AInStart, AInStop, DInStart, DInStop) then
      continue;

    // activate first inputs only
    InputIsAcive := i in [000..004, 064..068];
    inputs[i].InputEnabled := InputIsAcive;
    TLineSeries(fmMain.Chart1.Series[i]).active := InputIsAcive;
    TLineSeries(fmMain.Chart1.Series[i]).repaint;
    inputs[i].LineColor := Chart_GetSerieFixedColor(i);
    TLineSeries(fmMain.Chart1.Series[i]).SeriesColor := inputs[i].LineColor;

  end;

end;

function Chart_GetSerieFixedColor(i: integer): TColor;
begin
  case i of
    000, 063 : result := clLime;
    001, 064 : result := clRed;
    002, 065 : result := clYellow;
    003, 066 : result := clFuchsia;
    004, 067 : result := clBlue;
  else result := clGreen;
  end;

end;


procedure Chart_InsertStartMarker;
var
  i, j, VPos, AInStart, AInStop, DInStart, DInStop: integer;
  InputHandled, InputIsAnalog, InputIsDigital: boolean;
  EnabledInputCount: integer;
  YForceVal: double;
begin

  // get valid interval
  fmMain.HWInterface_GetValidInputs(AInStart, AInStop, DInStart, DInStop);

  for j := 1 to 3 do    // add 3 point to make a Pos Edge to Neg edge
  begin
    case j of
      1 : YForceVal := 0.0;
      2 : YForceVal := 1.0;
      3 : YForceVal := 0.0;
    end;

    // count enabled inputs
    EnabledInputCount := 0;

    // mark time
    LastTimePosition := now;

    // add analog and digital values
    for i := 127 downTo 0 do
    begin

      // input handled by actual interface ?
      if not fmMain.HWInterface_InputIsHandled(i, AInStart, AInStop, DInStart, DInStop) then
        continue;

      // input disabled ?
      if not Inputs[i].InputEnabled then
        continue;

      // count active inputs
      inc(EnabledInputCount);
      VPos := EnabledInputCount;

      // make transition
      case i of
        000..063 : Chart_AddXYAnalogPoint(i, VPos, LastTimePosition, 0.0, true, YForceVal, false);  // analog
        064..127 : Chart_AddXYDigitalPoint(i, VPos, LastTimePosition, 0.0, true, YForceVal, false);  // digital
      end;

    end;
  end;

  Chart_Repaint;

end;

procedure Chart_AddXYAnalogPoint(idx, VPos: integer; XVal: TDateTime; YVal: double; bForce: boolean; YForceVal: double; bUseInput: boolean);
var
  X, Y: double;
begin
  try

    // use YForceVal specified value
    if bForce then begin
      X := XVal;
      Y := YForceVal;

    // use array of inputs value
    end else if bUseInput then begin
      X := XVal;
      Y := Inputs[idx].actualValue;

    // use YVal specified value
    end else begin
      X := XVal;
      Y := YVal;

    end;

    // add point to chart
    fmMain.chart1.SeriesList[idx].AddXY(X, Y + VPos, '', Inputs[idx].LineColor);

  except
  end;
end;

procedure Chart_AddXYDigitalPoint(idx, VPos: integer; XVal: TDateTime; YVal: double; bForce: boolean; YForceVal: double; bUseInput: boolean);
var
  X, Y: double;
  b: boolean;
  val: double;
begin
  try

    // use YForceVal specified value
    if bForce then begin
      X := XVal;
      b := boolean(trunc(YForceVal));

    // use array of inputs value
    end else if bUseInput then begin
      X := XVal;
      b := Inputs[idx].actualState;

    // use YVal specified value
    end else begin
      X := XVal;
      b := boolean(trunc(YVal + 0.2));

    end;

    if b then
      Y := integer(b) - 0.1
    else
      Y := integer(b) + 0.01;

    // add point to chart
    fmMain.chart1.SeriesList[idx].AddXY(X, Y + VPos, '', Inputs[idx].LineColor);
  except
  end;
end;

procedure Chart_RefreshSeries;
var
  i: integer;
  s: string;
  AInStart, AInStop, DInStart, DInStop: integer;
  InputHandled, InputIsAcive, InputIsAnalog, InputIsDigital: boolean;
begin

  // get valid interval
  fmMain.HWInterface_GetValidInputs(AInStart, AInStop, DInStart, DInStop);

  // refresh drawing of chart
  for i := 0 to 127 do
  begin
    // is serie is TLineSeries ?
    if not (fmMain.Chart1.Series[i] is TLineSeries) then
      continue;

    TLineSeries(fmMain.Chart1.Series[i]).active := false;
    Case i of
      000..063 : s := format('AIn_%.3d', [i]);
      064..127 : s := format('DIn_%.3d', [i - 64]);
    end;
    TLineSeries(fmMain.Chart1.Series[i]).title := s;

    // input handled by actual interface ?
    if not fmMain.HWInterface_InputIsHandled(i, AInStart, AInStop, DInStart, DInStop) then
      continue;

    // input enabled ?
    if not inputs[i].InputEnabled then
      continue;

    // refresh state and labels
    InputIsAcive := inputs[i].InputEnabled;
    TLineSeries(fmMain.Chart1.Series[i]).active := InputIsAcive;
    TLineSeries(fmMain.Chart1.Series[i]).title := inputs[i].caption;
    TLineSeries(fmMain.Chart1.Series[i]).colorEachPoint := false;
    TLineSeries(fmMain.Chart1.Series[i]).SeriesColor := inputs[i].LineColor;
    TLineSeries(fmMain.Chart1.Series[i]).repaint;

  end;

end;

procedure Chart_Clear;
var
  i, j: integer;
begin
  for i := 0 to 127 do
  begin
    Inputs[i].WasInAlarm := false;

    // is serie is TLineSeries ?
    if not (fmMain.Chart1.Series[i] is TLineSeries) then
      continue;

    //fmMain.Chart1.Series[i].clear;
    for j := pred(fmMain.Chart1.Series[i].XValues.Count) downto 0 do
    begin
      fmMain.Chart1.Series[i].XValues.delete(j);
      fmMain.Chart1.Series[i].YValues.delete(j);
    end;
  end;

  // Chart_FillWithNullValues(1);


  Chart_Repaint;
end;

procedure Chart_Repaint;
begin
  // only refresh drawing, without erase list
  fmMain.chart1.Repaint;
end;

procedure Chart_Print;
begin
  with fmMain.PrintDialog1 do
  begin
    if execute then
      fmMain.chart1.print;
  end;
end;

procedure Chart_CheckForSave;
begin

end;

end.
