unit uDataRecord;

interface

uses
  Classes, SysUtils, Dialogs, Graphics;

function  Data_GetNewFileName: string;
procedure Data_SaveToFile(const sFile: string);
procedure Data_LoadFromFile(const sFile: string; EraseOldPoints: boolean);
procedure Data_RecordToFile;
procedure Data_AutoRecordToDisk;


implementation

uses
  rmMisc,
  uChartSeries,
  frmMain;


function Data_GetNewFileName: string;
var
  sDate, sNum, sExt, sFile: string;
  i: integer;
  bFileAlreadyExists: boolean;
begin
  sDate := FormatDateTime('dd_mm_yyyy_', now);
  sExt := '.dat';
  i := 0;
  repeat
    inc(i);
    sNum := format('%.4d', [i]);
    sFile := DataRecPath + sDate + sNum + sExt;
    bFileAlreadyExists := fileExists(sFile);
  until (not bFileAlreadyExists) or (i > 9999);
  if (not bFileAlreadyExists) and (i <=9999) then
    result := sFile
  else
    result := '';
end;

procedure Data_SaveToFile(const sFile: string);
var
  s: string;
  YVal, XVal: double;
  SeriesIndex, ValuesIndex: integer;
  SoftVersionNumber: string;
  lst: TStringList;
  VPos, EnabledInputCount: integer;
begin

  lst := TStringList.Create;
  try

    // first line must be Info string containing soft name and version number
    s := 'ADDATACQ VER 2.0';
    //s := format('%s VER %d', [SoftName, SoftVersionNumber]);
    lst.Add(s);

    // reset enabled inputs count
    EnabledInputCount := 0;

    // save input config used
    // to do

    // save color specified for series
    for SeriesIndex := 0 to 127 do begin
      if fmMain.chart1.seriesList[SeriesIndex].Active then begin
        s := format('COLOR_%3s=%d', [intToStr(SeriesIndex), inputs[SeriesIndex].lineColor]);
        lst.add(s);
      end;
    end;

    // save titles specified for series
    for SeriesIndex := 0 to 127 do begin
      if fmMain.chart1.seriesList[SeriesIndex].Active then begin
        s := format('TITLE_%3s=%s', [intToStr(SeriesIndex), inputs[SeriesIndex].caption]);
        lst.add(s);
      end;
    end;

    // save all values one by one
    for SeriesIndex := 127 downTo 0 do begin

      // input enabled ?
      if not fmMain.chart1.seriesList[SeriesIndex].Active then
        continue;

      // count active series
      inc(EnabledInputCount);
      VPos := EnabledInputCount;

      for ValuesIndex := 0 to pred(fmMain.chart1.seriesList[SeriesIndex].XValues.count) do begin
        XVal := fmMain.chart1.seriesList[SeriesIndex].XValues[ValuesIndex];
        YVal := fmMain.chart1.seriesList[SeriesIndex].YValues[ValuesIndex] - VPos;
        s := format('%d;%8.15f;%5.3f;', [SeriesIndex, XVal, YVal]);
        lst.Add(s);
      end;
    end;

    // save to file
    lst.SaveToFile(sFile);

  finally
    lst.Free;
  end;

  // file variable and main title refresh
  OpenedDataFileName := sFile;
  fmMain.SetMainAppTitle(OpenedDataFileName);
  ActualDataModified := false;
end;

procedure Data_LoadFromFile(const sFile: string; EraseOldPoints: boolean);

  function ExtractValues(s: string; var SeriesIndex: integer; var XVal, YVal: double): boolean;
  var
    sTmp: string;
    i, sep: integer;
  begin
    result := false;
    sTmp := s;

    for i := 1 to 3 do begin
      sep := pos(';', sTmp);
      if sep < 1 then  // error !
        exit;
      try
        case i of
          1 : SeriesIndex := strToInt(copy(sTmp, 1, sep - 1));  // series index
          2 : XVal := strToFloat(copy(sTmp, 1, sep - 1));      // DateTime stamp
          3 : YVal := strToFloat(copy(sTmp, 1, sep - 1));      // value
        end;
      except
        exit;
      end;
      delete(sTmp, 1, sep);
    end;

    result := true;
  end;

  function ExtractColor(s: string; var SeriesIndex: integer; var cl: TColor): boolean;
  var
    sTmp: string;
    sep: integer;
  begin
    result := false;
    sTmp := s;

    // delete COLOR_ string
    delete(sTmp, 1, 6);

    // get index and specified color
    sep := pos('=', sTmp);
    if sep < 1 then  // error !
      exit;
    try
      SeriesIndex := strToInt(copy(sTmp, 1, sep - 1));  // series index
      delete(sTmp, 1, sep);
      cl := strToInt(sTmp);      // value
    except
      exit;
    end;

    result := true;
  end;

  function ExtractTitle(s: string; var SeriesIndex: integer; var sTitle: string): boolean;
  var
    sTmp: string;
    sep: integer;
  begin
    result := false;
    sTmp := s;

    // delete TITLE_ string
    delete(sTmp, 1, 6);

    // get index and specified title
    sep := pos('=', sTmp);
    if sep < 1 then  // error !
      exit;
    try
      SeriesIndex := strToInt(copy(sTmp, 1, sep - 1));  // series index
      delete(sTmp, 1, sep);
      sTitle := sTmp;      // value
    except
      exit;
    end;

    result := true;
  end;

var
  s, sTmp, sTitle: string;
  i, InputIndex: integer;
  YVal, XVal: double;
  sep, SeriesIndex, ValuesIndex: integer;
  IsInfo: boolean;
  lst: TStringList;
  VPos, EnabledInputCount: integer;
  cl: TColor;
begin

  lst := TStringList.Create;
  try

    // load data file
    lst.loadFromFile(sFile);

    // first line must be Info string containing soft name and version number
    // if not, stop loading and exit load procedure
    s := 'ADDATACQ VER 2.0';
    //s := format('%s VER %f', [SoftName, SoftVersionNumber]);
    if not (lst[0] = s) then begin
      lst.free;
      exit;
    end;

    // clear old values if open new file,
    // keep old values if insert new file
    if EraseOldPoints then
      Chart_Clear;

    // reset alarms
    fmMain.ActAlarmResetCountExecute(nil);

    // disable all series
    for i := 0 to 127 do begin
      fmMain.chart1.SeriesList[i].Active := false;
      Inputs[i].WasInAlarm := false;
    end;

    // reset enabled inputs count
    EnabledInputCount := 0;

    // read all values one by one
    for i := 1 to pred(lst.count) do
    begin

      // read values
      s := lst[i];

      // extract color
      if system.Copy(s, 1, 6) = 'COLOR_' then begin

        if ExtractColor(s, SeriesIndex, cl) then begin
          Inputs[SeriesIndex].LineColor := cl;
          fmMain.Chart1.Series[i].colorEachPoint := false;
          fmMain.Chart1.Series[i].SeriesColor := cl;
        end;

      // extract title
      end else if system.Copy(s, 1, 6) = 'TITLE_' then begin

        if ExtractTitle(s, SeriesIndex, sTitle) then begin
          Inputs[SeriesIndex].caption := sTitle;
          fmMain.chart1.SeriesList[SeriesIndex].Title := sTitle;
        end;

      // extract values from string
      end else begin

        if ExtractValues(s, SeriesIndex, XVal, YVal) then begin

          // active serie and count
          if not fmMain.chart1.SeriesList[SeriesIndex].Active then begin
            inputs[SeriesIndex].InputEnabled := true;
            fmMain.chart1.SeriesList[SeriesIndex].Active := true;
            inc(EnabledInputCount);
            VPos := EnabledInputCount;
          end;

          case SeriesIndex of
            000..063 : Chart_AddXYAnalogPoint(SeriesIndex, VPos, XVal, YVal, false, 0.0, false);
            064..127 : Chart_AddXYDigitalPoint(SeriesIndex, VPos, XVal, YVal, false, 0.0, false);
          end;

        end;
      end;
    end;

  finally
    lst.Free;
  end;

  Chart_RefreshSeries;
  Chart_Repaint;

  // file variable and main title refresh
  OpenedDataFileName := sFile;
  fmMain.SetMainAppTitle(OpenedDataFileName);
  ActualDataModified := false;

end;

procedure Data_RecordToFile;
var
  iBefore, iAfter, iMax,
  FromFirstStartEllapsed, FromLastEndEllapsed,
  PointsCount: integer;
  bChartMaxPointsReached, DoRec: boolean;
  sRec: string;
begin

  with UserPrefs, AutoRecordToDisk do
  begin
    iMax := ChartMaxPoints;
    iBefore := ChartMaxPointsBeforeFirstAlarm;
    iAfter := ChartMaxPointsAfterLastAlarm;
    FromFirstStartEllapsed := CountFromStartOfFirstAlarm;
    FromLastEndEllapsed := CountFromEndOfLasttAlarm;
    PointsCount := iBefore + FromFirstStartEllapsed + iAfter;
    bChartMaxPointsReached := PointsCount >= iMax;
    DoRec := bChartMaxPointsReached;
  end;

  if DoRec then begin
    Data_AutoRecordToDisk;
  end;

end;

procedure Data_AutoRecordToDisk;
var
  sRec: string;
begin
  with AutoRecordToDisk do begin

    // get a new file name for new file
    FileName := Data_GetNewFileName;

    // save to file
    Data_SaveToFile(FileName);

    // reset variables used for autorecord on disk
    RecordInProgress := false;
    IsMainAlarm := false;
    // WasMainAlarm := false;
    CountFromStartOfFirstAlarm := 0;
    CountFromEndOfLasttAlarm := 0;

    // file variable and main title refresh
    OpenedDataFileName := FileName;
    fmMain.SetMainAppTitle(OpenedDataFileName);
    ActualDataModified := false;
  end;

  fmMain.StatusBar_RefreshRecordStatus;

end;


end.
