unit uLog;

interface

uses
  Forms, Controls, Classes, Comctrls, SysUtils, Dialogs, Graphics,
  uUserPrefs;

type

  TEventItem = record
    sEvent: string;
    iIdx: integer;
    dtDateTime: TDateTime;
    iDuration: integer;
  end;

procedure Log_Init;
function  Log_FormatButtonIndex(idx: integer): string;
function  Log_FormatDuration(sVal: string; iVal: integer): string;
function  Log_FormatDateTime(dt: TDateTime): string; overload;
function  Log_FormatDateTime(s: string): TDateTime; overload;
procedure Log_New(sFile: string);
procedure Log_SelectAll;
procedure Log_InvertSelection;
procedure Log_ClearAll;
procedure Log_ClearSelection;
function  Log_StringToEventItem(s: string; var EventItemTmp: TEventItem): boolean;
procedure Log_LoadFromFile(sFile: string);
procedure Log_SetOpenedLogFile(sFile: string);
procedure Log_SetOpenedLogModified(bState: boolean);
procedure Log_CheckForSave(bAskForSave: boolean);
procedure Log_AddLineToFile(sFile: string);
procedure Log_SaveToFile(sFile: string);
procedure Log_MakeBackup(sFile: string);
function  Log_AddLine(
            EventItem: TEventItem;
            bInsertState,
            bState: boolean;
            InsertAtBegin: boolean): integer;
procedure Log_RefreshLine(
            bEvent: boolean;
            bBtnIdx: boolean;
            bDateTime: boolean;
            bDuration: boolean);
procedure Log_ShowLogFile;
procedure Log_CheckForEventListNotEmpty;
function  Log_GetEventParams(
            RecPrefs: TUserPrefs;
            idx: integer;
            var bActive: boolean;
            var bInvert: boolean;
            var iMinDuration: integer;
            var sLabel: string;
            var clText: TColor;
            var clBG: TColor): boolean;
function  Log_SetEventParams(
            var RecPrefs: TUserPrefs;
            idx: integer;
            bActive: boolean;
            bInvert: boolean;
            iMinDuration: integer;
            sLabel: string;
            clText: TColor;
            clBG: TColor): boolean;
procedure Log_RefreshView;


var
  EventItem: TEventItem;

const
   LogIdxBtnIndex = 0;
   LogIdxDateTime = 1;
   LogIdxDuration = 2;

implementation

uses
  rmMisc,
  //strCon,
  uResStr,
  frmMain, frmReport01;

procedure Log_Init;
begin
  if FileExists(LogFileOfTheDay) then
    Log_LoadFromFile(LogFileOfTheDay)
  else
    Log_New(LogFileOfTheDay);
end;

function Log_FormatButtonIndex(idx: integer): string;
begin
  result := format('%d', [Idx]);
end;

function Log_FormatDuration(sVal: string; iVal: integer): string;
var
  iDuration: integer; // always in ms
  sDuration, sUnit: string;
begin
  // get value from string (string not empty)
  if (sVal <> '') then
  begin
    rmMisc.StrToKeyAndValue(trim(sVal), ' ', sDuration, sUnit);
    rmMisc.GetGoodDecimalSeparator(sDuration);
    sUnit := upperCase(sUnit);
    //showmessage(format('"%s" "%s" "%s"', [sVal, sDuration, sUnit]));
    if sUnit = 'MS' then
      iDuration := rmMisc.Str2Int(sDuration)
    else
      iDuration := trunc(rmMisc.StrToReal(sDuration) * 1000);
  end
  else

  // get value from integer (string is empty)
  begin
    iDuration := iVal;
  end;

  // format result
  case UserPrefs.LogDurationFormat of

    // always in ms
    0 : result := format('%d ms', [iDuration]);

    // always in s
    1 : result := format('%.3f s', [iDuration / 1000]);

    // in ms or s
    2 : begin
          if (iDuration >= 1000) then
            result := format('%.3f s', [iDuration / 1000])
          else
            result := format('%d ms', [iDuration]);
        end;
  end;
end;

function Log_FormatDateTime(dt: TDateTime): string;
begin
  try
    result := formatDateTime('dd/mm/yyyy hh:nn:ss', dt);
  except
  end;
end;

function Log_FormatDateTime(s: string): TDateTime;
begin
  try
    result := StrToDateTime(s);
  except
  end;
end;

procedure Log_New(sFile: string);
begin
  Log_CheckForSave(true);
  Log_ClearAll;
  Log_SetOpenedLogFile(sFile);
  Log_SetOpenedLogModified(False);
end;

procedure Log_SelectAll;
var
  i: integer;
begin
  with fmMain.lvEventsList do
  begin
    for i := pred(items.count) downTo 0 do
      items[i].Selected := true;
  end;
end;

procedure Log_InvertSelection;
var
  i: integer;
begin
  with fmMain.lvEventsList do
  begin
    for i := pred(items.count) downTo 0 do
      items[i].Selected := not items[i].Selected;
  end;
end;

procedure Log_ClearAll;
begin
  with fmMain.lvEventsList do
  begin
    Items.Clear;
  end;
  Log_SetOpenedLogModified(true);
end;

procedure Log_ClearSelection;
var
  i: integer;
  CountBefore, CountAfter: integer;
begin
  with fmMain.lvEventsList do
  begin
    for i := pred(items.count) downTo 0 do
    begin
      if items[i].Selected then
        items.delete(i);
    end;
  end;
  if CountBefore <> CountAfter then
    Log_SetOpenedLogModified(true);
end;

function Log_StringToEventItem(s: string; var EventItemTmp: TEventItem): boolean;
var
  lstTmp: TStringList;
  sTmp: string;
  Err: boolean;
begin
  if s = '' then
    exit;

  Err := false;
  sTmp := s;

  lstTmp := TStringList.create;
  try

    // Decode string
    rmMisc.StrToList(sTmp, ';', lstTmp, true);

    // extract values
    if lstTmp.Count > 3 then
    begin

      with EventItemTmp do
      begin

        try

          // get event
          sEvent := lstTmp[0];

          // get buton index
          sTmp := lstTmp[1];
          iIdx := StrToInt(sTmp);

          // get date-time
          sTmp := lstTmp[2];
          dtDateTime := Log_FormatDateTime(sTmp);

          // get duration
          sTmp := lstTmp[3];
          if pos('MS', upperCase(sTmp)) > 0 then
          begin
            sTmp := copy(sTmp, 1, pos('MS', upperCase(sTmp)) - 1);
            sTmp := trim(sTmp);
            iDuration := strToInt(sTmp);
          end
          else
          if pos('S', upperCase(sTmp)) > 0 then
          begin
            sTmp := copy(sTmp, 1, pos('S', upperCase(sTmp)) - 1);
            sTmp := trim(sTmp);
            iDuration := trunc(strToFloat(sTmp) * 1000);
          end
          else
          begin
            iDuration := strToInt(sTmp);
          end;

        except
          Err := true;
        end;
      end;
    end;

  finally
    lstTmp.free;
  end;

  result := not Err;
end;

procedure Log_LoadFromFile(sFile: string);
var
  EventItemTmp: TEventItem;
  F: TextFile;
  s: string;
  OK: boolean;
begin
  Log_ClearAll;
  try
    try
      AssignFile(F, sFile);
      Reset(F);
      while not EOF(F) do
      begin
        Readln(F, s);
        OK := Log_StringToEventItem(s, EventItemTmp);
        if OK then
          Log_AddLine(EventItemTmp, false, false, false);
      end;
    finally
      CloseFile(F);
    end;
  except
    if UserPrefs.MiscShowErrorMessages then
      MessageDlg(
        format(sErrLoadFromLogFileFailed, [extractFileName(sFile)]),
        mtInformation, [mbOK], 0);
  end;
  Log_SetOpenedLogFile(sFile);
  Log_SetOpenedLogModified(False);
end;

procedure Log_SetOpenedLogFile(sFile: string);
var
  sTmp: string;
begin
  LogOpenedFileName := sFile;

  // set main app title
  if sFile <> '' then
    sTmp := extractFileName(sFile)
  else
    sTmp := 'Nouveau';
  fmMain.Caption := format('%s - [%s]', [UserPrefs.AppMainTitle, sTmp])
end;

procedure Log_SetOpenedLogModified(bState: boolean);
begin
  //showmessage(booltostr(bstate,'true','false'));
  LogOpenedFileModified := bState;
  {
  if not CaptureIsRunning then
    fmMain.App_RefreshGUIStatusBar;
  }
end;

procedure Log_CheckForSave(bAskForSave: boolean);
var
  Ret: Integer;
begin
  if not LogOpenedFileModified then
    exit;
    
  // ask for save changes ?
  if bAskForSave then
    Ret := MessageDlg(sAskSaveChanges, mtConfirmation, mbYesNoCancel, 0)
  else
    // if userPrefs.LogSaveAutoOnExit then
    ret := mrYes;

  // save ?  
  case Ret of
    mrYes: fmMain.ActFileSaveExecute(nil);
    mrNo: ;  // no save, and continue
    mrCancel: Abort;
  end;
end;

procedure Log_SaveToFile(sFile: string);
var
  F: TextFile;
  s: string;
  i: integer;
begin
  // make backup ?
  if UserPrefs.LogSaveMakeBackup then
    Log_MakeBackup(sFile);

  try
    try
      AssignFile(F, sFile);
      Rewrite(F);
      with fmMain.lvEventsList do
      begin
        for i := 0 to pred(fmMain.lvEventsList.items.count) do
        begin
          // make string to write to file
          s := items[i].caption;
          s := s + ';' + items[i].SubItems[0];
          s := s + ';' + items[i].SubItems[1];
          s := s + ';' + items[i].SubItems[2];

          // write to file
          Writeln(F, s);
        end;
      end;

    finally
      CloseFile(F);
    end;

    Log_SetOpenedLogFile(sFile);
    Log_SetOpenedLogModified(False);
  except
    if UserPrefs.MiscShowErrorMessages and
    (not CaptureIsRunning) then
      MessageDlg(
      format(sErrSaveToLogFileFailed, [extractFileName(sFile)]),
      mtInformation, [mbOK], 0);
  end;
end;

procedure Log_AddLineToFile(sFile: string);
var
  F: TextFile;
  s: string;
  idx: integer;
begin
  try
    try
      AssignFile(F, sFile);
      if not fileExists(sFile) then
        Rewrite(F)
      else
        Append(F);
      with fmMain.lvEventsList do
      begin
        //idx := pred(fmMain.lvEventsList.items.count);
        idx := 0;

        // make line to write to file
        s := items[idx].caption;
        s := s + ';' + items[idx].SubItems[0];
        s := s + ';' + items[idx].SubItems[1];
        s := s + ';' + items[idx].SubItems[2];

        // write to file
        Writeln(F, s);
        Flush(F);

      end;
    finally
      CloseFile(F);
    end;
    Log_SetOpenedLogModified(False);

  except
  end;
end;

procedure Log_MakeBackup(sFile: string);
var
  sBackupFile: string;
begin
  if fileExists(sFile) then
  begin
    sBackupFile := changeFileExt(sFile, '.bak');
    try
      if fileExists(sBackupFile) then
        DeleteFile(sBackupFile);
      renameFile(sFile, sBackupFile);
    except
    end;
  end;
end;

function  Log_AddLine(
            EventItem: TEventItem;
            bInsertState,
            bState: boolean;
            InsertAtBegin: boolean): integer;
var
  i, idx: integer;
  bActive,
  bInvert: boolean;
  iMinDuration: integer;
  sLabel: string;
  clTextColor, clBGColor: TColor;
  NewItem: TListItem;
begin
  // min duration accepted = 1 ms
  {
  if EventItem.iDuration < 1 then
  begin
    exit;
  end;
  }

  // add new item in list view
  with fmMain.lvEventsList do
  begin
    try
      Selected := nil;

      // insert at begining or at end of list
      if InsertAtBegin then
        NewItem := items.Insert(0)
      else
        NewItem := items.Add;

      // set values
      if bInsertState then
        NewItem.caption := EventItem.sEvent + boolToStr(bState, ' [ON]', ' [OFF]')
      else
        NewItem.caption := EventItem.sEvent;
      NewItem.SubItems.Add(Log_FormatButtonIndex(EventItem.iIdx));
      NewItem.SubItems.Add(Log_FormatDateTime(EventItem.dtDateTime));
      NewItem.SubItems.Add(Log_FormatDuration('', EventItem.iDuration));
      result := NewItem.Index;
      //Selected := Items[0];
    except
    end;
  end;
  //Log_SetOpenedLogModified(true);

  // get all params, but use only text and BG colors
  Log_GetEventParams(
    UserPrefs,
    EventItem.iIdx,
    bActive,
    bInvert,
    iMinDuration,
    sLabel,
    clTextColor,
    clBGColor);

  // refresh large label (Last Event occured)
  with fmMain.lblLastEvent do
  begin
    caption := EventItem.sEvent;
    color := clBGColor;
    font.color := clTextColor;
  end;

  fmMain.ActLogRefreshExecute(nil);
end;

procedure Log_RefreshLine(
            bEvent: boolean;
            bBtnIdx: boolean;
            bDateTime: boolean;
            bDuration: boolean);
var
  i: integer;
  sOld, sNew: string;
begin
  with fmMain.lvEventsList do
  begin
    for i := 0 to pred(items.count) do
    begin

      // refresh duration ?
      if bDuration then
      begin
        sOld := items[i].subItems[LogIdxDuration];
        sNew := Log_FormatDuration(sOld, -1);
        items[i].subItems[LogIdxDuration] := sNew;
      end;

    end;
  end;
  Log_SetOpenedLogModified(true);
end;

procedure Log_ShowLogFile;
begin
  try

  except
  end;
end;

procedure Log_CheckForEventListNotEmpty;
begin
  if fmMain.lvEventsList.Items.count = 0 then
  begin
    MessageDlg(sNoEventToHandle, mtInformation, [mbOK], 0);
    abort;
  end;
end;

function Log_GetEventParams(
           RecPrefs: TUserPrefs;
           idx: integer;
           var bActive: boolean;
           var bInvert: boolean;
           var iMinDuration: integer;
           var sLabel: string;
           var clText: TColor;
           var clBG: TColor): boolean;
begin
  //showmessage(inttostr(idx));
  with RecPrefs do
  begin
    case idx of
      0 : begin
            bActive := LogEvent0Active;
            bInvert := LogEvent0Invert;
            iMinDuration := LogEvent0MinDuration;
            sLabel := LogEvent0Label;
            clText := LogEvent0TextColor;
            clBG := LogEvent0BGColor;
          end;
      1 : begin
            bActive := LogEvent1Active;
            bInvert := LogEvent1Invert;
            iMinDuration := LogEvent1MinDuration;
            sLabel := LogEvent1Label;
            clText := LogEvent1TextColor;
            clBG := LogEvent1BGColor;
          end;
      2 : begin
            bActive := LogEvent2Active;
            bInvert := LogEvent2Invert;
            iMinDuration := LogEvent2MinDuration;
            sLabel := LogEvent2Label;
            clText := LogEvent2TextColor;
            clBG := LogEvent2BGColor;
          end;
      3 : begin
            bActive := LogEvent3Active;
            bInvert := LogEvent3Invert;
            iMinDuration := LogEvent3MinDuration;
            sLabel := LogEvent3Label;
            clText := LogEvent3TextColor;
            clBG := LogEvent3BGColor;
          end;
      4 : begin
            bActive := LogEvent4Active;
            bInvert := LogEvent4Invert;
            iMinDuration := LogEvent4MinDuration;
            sLabel := LogEvent4Label;
            clText := LogEvent4TextColor;
            clBG := LogEvent4BGColor;
          end;
      5 : begin
            bActive := LogEvent5Active;
            bInvert := LogEvent5Invert;
            iMinDuration := LogEvent5MinDuration;
            sLabel := LogEvent5Label;
            clText := LogEvent5TextColor;
            clBG := LogEvent5BGColor;
          end;
      6 : begin
            bActive := LogEvent6Active;
            bInvert := LogEvent6Invert;
            iMinDuration := LogEvent6MinDuration;
            sLabel := LogEvent6Label;
            clText := LogEvent6TextColor;
            clBG := LogEvent6BGColor;
          end;
      7 : begin
            bActive := LogEvent7Active;
            bInvert := LogEvent7Invert;
            iMinDuration := LogEvent7MinDuration;
            sLabel := LogEvent7Label;
            clText := LogEvent7TextColor;
            clBG := LogEvent7BGColor;
          end;
      8 : begin
            bActive := LogEvent8Active;
            bInvert := LogEvent8Invert;
            iMinDuration := LogEvent8MinDuration;
            sLabel := LogEvent8Label;
            clText := LogEvent8TextColor;
            clBG := LogEvent8BGColor;
          end;
      9 : begin
            bActive := LogEvent9Active;
            bInvert := LogEvent9Invert;
            iMinDuration := LogEvent9MinDuration;
            sLabel := LogEvent9Label;
            clText := LogEvent9TextColor;
            clBG := LogEvent9BGColor;
          end;
      10 : begin
            bActive := LogEvent10Active;
            bInvert := LogEvent10Invert;
            iMinDuration := LogEvent10MinDuration;
            sLabel := LogEvent10Label;
            clText := LogEvent10TextColor;
            clBG := LogEvent10BGColor;
          end;
      11 : begin
            bActive := LogEvent11Active;
            bInvert := LogEvent11Invert;
            iMinDuration := LogEvent11MinDuration;
            sLabel := LogEvent11Label;
            clText := LogEvent11TextColor;
            clBG := LogEvent11BGColor;
          end;
      12 : begin
            bActive := LogEvent12Active;
            bInvert := LogEvent12Invert;
            iMinDuration := LogEvent12MinDuration;
            sLabel := LogEvent12Label;
            clText := LogEvent12TextColor;
            clBG := LogEvent12BGColor;
          end;
      {
      13 : begin
            bActive := LogEvent13Active;
            bInvert := LogEvent13Invert;
            iMinDuration := LogEvent13MinDuration;
            sLabel := LogEvent13Label;
            clText := LogEvent13TextColor;
            clBG := LogEvent13BGColor;
          end;
      14 : begin
            bActive := LogEvent14Active;
            bInvert := LogEvent14Invert;
            iMinDuration := LogEvent14MinDuration;
            sLabel := LogEvent14Label;
            clText := LogEvent14TextColor;
            clBG := LogEvent14BGColor;
          end;
      15 : begin
            bActive := LogEvent15Active;
            bInvert := LogEvent15Invert;
            iMinDuration := LogEvent15MinDuration;
            sLabel := LogEvent15Label;
            clText := LogEvent15TextColor;
            clBG := LogEvent15BGColor;
          end;
      16 : begin
            bActive := LogEvent16Active;
            bInvert := LogEvent16Invert;
            iMinDuration := LogEvent16MinDuration;
            sLabel := LogEvent16Label;
            clText := LogEvent16TextColor;
            clBG := LogEvent16BGColor;
          end;
      }
      else begin
            bActive := false;
            bInvert := false;
            iMinDuration := 2;
            sLabel := '';
            clText := clBlack;
            clBG := clWindow;
          end;
    end;
  end;
end;

function Log_SetEventParams(
           var RecPrefs: TUserPrefs;
           idx: integer;
           bActive: boolean;
           bInvert: boolean;
           iMinDuration: integer;
           sLabel: string;
           clText: TColor;
           clBG: TColor): boolean;
begin
  with RecPrefs do
  begin
    case idx of
      0 : begin
            LogEvent0Active := bActive;
            LogEvent0Invert := bInvert;
            LogEvent0MinDuration := iMinDuration;
            LogEvent0Label := sLabel;
            LogEvent0TextColor := clText;
            LogEvent0BGColor := clBG;
          end;
      1 : begin
            LogEvent1Active := bActive;
            LogEvent1Invert := bInvert;
            LogEvent1MinDuration := iMinDuration;
            LogEvent1Label := sLabel;
            LogEvent1TextColor := clText;
            LogEvent1BGColor := clBG;
          end;
      2 : begin
            LogEvent2Active := bActive;
            LogEvent2Invert := bInvert;
            LogEvent2MinDuration := iMinDuration;
            LogEvent2Label := sLabel;
            LogEvent2TextColor := clText;
            LogEvent2BGColor := clBG;
          end;
      3 : begin
            LogEvent3Active := bActive;
            LogEvent3Invert := bInvert;
            LogEvent3MinDuration := iMinDuration;
            LogEvent3Label := sLabel;
            LogEvent3TextColor := clText;
            LogEvent3BGColor := clBG;
          end;
      4 : begin
            LogEvent4Active := bActive;
            LogEvent4Invert := bInvert;
            LogEvent4MinDuration := iMinDuration;
            LogEvent4Label := sLabel;
            LogEvent4TextColor := clText;
            LogEvent4BGColor := clBG;
          end;
      5 : begin
            LogEvent5Active := bActive;
            LogEvent5Invert := bInvert;
            LogEvent5MinDuration := iMinDuration;
            LogEvent5Label := sLabel;
            LogEvent5TextColor := clText;
            LogEvent5BGColor := clBG;
          end;
      6 : begin
            LogEvent6Active := bActive;
            LogEvent6Invert := bInvert;
            LogEvent6MinDuration := iMinDuration;
            LogEvent6Label := sLabel;
            LogEvent6TextColor := clText;
            LogEvent6BGColor := clBG;
          end;
      7 : begin
            LogEvent7Active := bActive;
            LogEvent7Invert := bInvert;
            LogEvent7MinDuration := iMinDuration;
            LogEvent7Label := sLabel;
            LogEvent7TextColor := clText;
            LogEvent7BGColor := clBG;
          end;
      8 : begin
            LogEvent8Active := bActive;
            LogEvent8Invert := bInvert;
            LogEvent8MinDuration := iMinDuration;
            LogEvent8Label := sLabel;
            LogEvent8TextColor := clText;
            LogEvent8BGColor := clBG;
          end;
      9 : begin
            LogEvent9Active := bActive;
            LogEvent9Invert := bInvert;
            LogEvent9MinDuration := iMinDuration;
            LogEvent9Label := sLabel;
            LogEvent9TextColor := clText;
            LogEvent9BGColor := clBG;
          end;
      10 : begin
            LogEvent10Active := bActive;
            LogEvent10Invert := bInvert;
            LogEvent10MinDuration := iMinDuration;
            LogEvent10Label := sLabel;
            LogEvent10TextColor := clText;
            LogEvent10BGColor := clBG;
          end;
      11 : begin
            LogEvent11Active := bActive;
            LogEvent11Invert := bInvert;
            LogEvent11MinDuration := iMinDuration;
            LogEvent11Label := sLabel;
            LogEvent11TextColor := clText;
            LogEvent11BGColor := clBG;
          end;
      12 : begin
            LogEvent12Active := bActive;
            LogEvent12Invert := bInvert;
            LogEvent12MinDuration := iMinDuration;
            LogEvent12Label := sLabel;
            LogEvent12TextColor := clText;
            LogEvent12BGColor := clBG;
          end;
      {
      13 : begin
            LogEvent13Active := bActive;
            LogEvent13Invert := bInvert;
            LogEvent13MinDuration := iMinDuration;
            LogEvent13Label := sLabel;
            LogEvent13TextColor := clText;
            LogEvent13BGColor := clBG;
          end;
      14 : begin
            LogEvent14Active := bActive;
            LogEvent14Invert := bInvert;
            LogEvent14MinDuration := iMinDuration;
            LogEvent14Label := sLabel;
            LogEvent14TextColor := clText;
            LogEvent14BGColor := clBG;
          end;
      15 : begin
            LogEvent15Active := bActive;
            LogEvent15Invert := bInvert;
            LogEvent15MinDuration := iMinDuration;
            LogEvent15Label := sLabel;
            LogEvent15TextColor := clText;
            LogEvent15BGColor := clBG;
          end;
      16 : begin
            LogEvent16Active := bActive;
            LogEvent16Invert := bInvert;
            LogEvent16MinDuration := iMinDuration;
            LogEvent16Label := sLabel;
            LogEvent16TextColor := clText;
            LogEvent16BGColor := clBG;
          end;
      }
    end;
    //showmessage(LogEvent1Label);
  end;
end;

procedure Log_RefreshView;
begin
  fmMain.lvEventsList.Refresh;
end;


end.
